class
	P_MIXER_STATIC

inherit
	CE_UTILITIES
	S_STDLIB_STATIC
	S_SDL_SDL_ERROR_STATIC
	S_STRING_STATIC
	S_SDL_SDL_TIMER_STATIC
	S_SDL_SDL_AUDIO_STATIC
	S_SDL_SDL_RWOPS_STATIC

feature {ANY} -- public attributes

	audio_opened: CE_STATIC [INTEGER_32]
		once ("PROCESS")
			Result.item := 0
		end

	mixer: CE_STATIC [CE_ARRAY [SD_SDL_AUDIOSPEC]]
		once ("PROCESS")
			Result.item := create {CE_CLASS_ARRAY [SD_SDL_AUDIOSPEC]}.make (<<1>>)
		end

	mix_channel: CE_STATIC [CE_POINTER [PD_MIX_CHANNEL]]
		once ("PROCESS")
			Result.item := create {CE_CLASS_POINTER [PD_MIX_CHANNEL]}.make_shared (default_pointer)
		end

	posteffects: CE_STATIC [CE_ARRAY [CE_POINTER [PD_MIX_EFFECTINFO]]]
		once ("PROCESS")
			Result.item := create {CE_POINTER_ARRAY [PD_MIX_EFFECTINFO]}.make_initialized (<<
					create {CE_CLASS_POINTER [PD_MIX_EFFECTINFO]}.make_shared (default_pointer)
				>>)
		end

	num_channels: CE_STATIC [INTEGER_32]
		once ("PROCESS")
		end

	reserved_channels: CE_STATIC [INTEGER_32]
		once ("PROCESS")
			Result.item := 0
		end

	mix_postmix: CE_STATIC [CE_ROUTINE]
		once ("PROCESS")
			Result.item := create {CE_ROUTINE}.make_shared (default_pointer)
		end

	mix_postmix_data: CE_STATIC [CE_POINTER [ANY]]
		once ("PROCESS")
			Result.item := create {CE_VOID_POINTER}.make_shared (default_pointer)
		end

	channel_done_callback: CE_STATIC [CE_ROUTINE]
		once ("PROCESS")
			Result.item := create {CE_ROUTINE}.make_shared (default_pointer)
		end

	mix_music: CE_STATIC [CE_ROUTINE]
		once ("PROCESS")
			Result.item := (create {P_MUSIC_STATIC}).music_mixer_address
		end

	music_data: CE_STATIC [CE_POINTER [ANY]]
		once ("PROCESS")
			Result.item := create {CE_VOID_POINTER}.make_shared (default_pointer)
		end

	chunk_decoders: CE_STATIC [CE_POINTER [CE_POINTER [INTEGER_8]]]
		once ("PROCESS")
			Result.item := create {CE_POINTER_POINTER [INTEGER_8]}.make_shared (default_pointer)
		end

	num_decoders: CE_STATIC [INTEGER_32]
		once ("PROCESS")
			Result.item := 0
		end

	linked_version: CE_STATIC [CE_ARRAY [SD_SDL_VERSION]]
		once ("PROCESS")
			Result.item := create {CE_CLASS_ARRAY [SD_SDL_VERSION]}.make (<<1>>)
		end

	initialized: CE_STATIC [INTEGER_32]
		once ("PROCESS")
			Result.item := 0
		end

feature {ANY} -- public routines

	mix_get_num_chunk_decoders: INTEGER_32
		do
			Result := num_decoders.item
		end

	mix_get_chunk_decoder (a_index: INTEGER_32): CE_POINTER [INTEGER_8]
		local
			l_return: BOOLEAN
		do
			if a_index < 0 then
				Result := (create {CE_INTEGER_8_POINTER}.make_shared (default_pointer))
				l_return := True
			elseif a_index >= num_decoders.item then
				Result := (create {CE_INTEGER_8_POINTER}.make_shared (default_pointer))
				l_return := True
			end
			if not l_return then
				Result := (chunk_decoders.item[a_index])
			end
		end

	add_chunk_decoder (a_decoder: CE_POINTER [INTEGER_8])
		local
			l_ptr: CE_POINTER [ANY]
			l_tmp_0: INTEGER_32
			l_return: BOOLEAN
		do
			l_ptr := (realloc (create {CE_VOID_POINTER}.make_cast (chunk_decoders.item), (num_decoders.item + 1).to_natural_32 * {PLATFORM}.Pointer_bytes.to_natural_32))
			if l_ptr.is_void then
				l_return := True
			end
			if not l_return then
				chunk_decoders.item := create {CE_POINTER_POINTER [INTEGER_8]}.make_cast (l_ptr)
				l_tmp_0 := num_decoders.item
				num_decoders.item := num_decoders.item + 1
				chunk_decoders.item[l_tmp_0] := a_decoder
			end
		end

	mix_linked_version: CE_POINTER [SD_SDL_VERSION]
		do
			linked_version.item.item.major := (1).to_natural_8
			linked_version.item.item.minor := (2).to_natural_8
			linked_version.item.item.patch := (12).to_natural_8
			Result := linked_version.item
		end

	mix_init (a_flags: INTEGER_32): INTEGER_32
		local
			l_result: INTEGER_32
		do
			l_result := 0
			if ((a_flags).bit_and (16)).to_boolean then
				sdl_set_error ([ce_string ("Mixer not built with FluidSynth support")])
			end
			if ((a_flags).bit_and (1)).to_boolean then
				sdl_set_error ([ce_string ("Mixer not built with FLAC support")])
			end
			if ((a_flags).bit_and (2)).to_boolean then
				sdl_set_error ([ce_string ("Mixer not built with MOD support")])
			end
			if ((a_flags).bit_and (4)).to_boolean then
				if ((initialized.item).bit_and (4)).to_boolean then
					l_result := (l_result).bit_or ((4))
				elseif ((create {P_DYNAMIC_MP3_STATIC}).mix_init_mp3) = 0 then
					l_result := (l_result).bit_or ((4))
				end
			end
			if ((a_flags).bit_and (8)).to_boolean then
				sdl_set_error ([ce_string ("Mixer not built with Ogg Vorbis support")])
			end
			initialized.item := (initialized.item).bit_or (l_result)
			Result := l_result
		end

	mix_quit
		do
			if ((initialized.item).bit_and (4)).to_boolean then
				(create {P_DYNAMIC_MP3_STATIC}).mix_quit_mp3
			end
			initialized.item := 0
		end

	mix_channel_done_playing (a_channel: INTEGER_32)
		do
			if (not channel_done_callback.item.is_void) then
				channel_done_callback.item.call ([a_channel])
			end
			mix_remove_all_effects (a_channel, create {CE_POINTER_POINTER [PD_MIX_EFFECTINFO]}.make_shared ((mix_channel.item + a_channel).c_pointer + (create {PD_MIX_CHANNEL}.make).offset_of ("effects"))).do_nothing
		end

	mix_do_effects (a_chan: INTEGER_32; a_snd: CE_POINTER [ANY]; a_len: INTEGER_32): CE_POINTER [ANY]
		local
			l_posteffect: INTEGER_32
			l_e: CE_POINTER [PD_MIX_EFFECTINFO]
			l_tmp: CE_POINTER [PD_MIX_EFFECTINFO]
			l_buf: CE_POINTER [ANY]
			l_return: BOOLEAN
		do
			l_posteffect := (a_chan = -2).to_integer
			if l_posteffect.to_boolean then
				l_tmp := posteffects.item.item
			else
				l_tmp := (mix_channel.item + a_chan).item.effects
			end
			l_e := l_tmp
			l_buf := a_snd
			if not l_e.is_void then
				if not l_posteffect.to_boolean then
					l_buf := malloc (a_len.to_natural_32)
					if l_buf.is_void then
						Result := a_snd
						l_return := True
					end
					if not l_return then
						memcpy (l_buf, a_snd, a_len.to_natural_32).do_nothing
					end
				end
				if not l_return then
					from
					until
						l_e.is_void
					loop
						if not (l_e.item.callback).is_void then
							(l_e.item.callback).call ([a_chan, l_buf, a_len, l_e.item.udata])
						end
						l_e := l_e.item.next
					end
				end
			end
			if not l_return then
				Result := l_buf
			end
		end

	mix_channels (a_udata: CE_POINTER [ANY]; a_stream: CE_POINTER [NATURAL_8]; a_len: INTEGER_32)
		local
			l_mix_input: CE_POINTER [NATURAL_8]
			l_i: INTEGER_32
			l_mixable: INTEGER_32
			l_volume: INTEGER_32
			l_sdl_ticks: NATURAL_32
			l_ticks: NATURAL_32
			l_index: INTEGER_32
			l_remaining: INTEGER_32
			l_alen: INTEGER_32
			l_return, l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_volume := 128
					if ((create {P_MUSIC_STATIC}).music_active.item).to_boolean then
						mix_music.item.call ([music_data.item, a_stream, a_len])
					elseif not mix_music.item.is_equal ((create {P_MUSIC_STATIC}).music_mixer_address) then
						mix_music.item.call ([music_data.item, a_stream, a_len])
					end
					l_sdl_ticks := sdl_get_ticks
					l_i := 0
				when 1 then -- loop condition:
					if not (not (l_return) and then (l_i < num_channels.item)) then
						l_pos := -1 + 9 -- skip loop
					end
				when 2 then -- loop body:
						if ((mix_channel.item + l_i).item.paused).to_boolean then
							l_pos := -1 + 8 -- skip if block
						end
				when 3 then -- if:
						if not ((mix_channel.item + l_i).item.expire > {NATURAL_32} 0) then
							l_pos := -1 + 6 -- skip if block
						end
				when 4 then -- if:
						if (mix_channel.item + l_i).item.expire < l_sdl_ticks then
							(mix_channel.item + l_i).item.playing := 0;
							(mix_channel.item + l_i).item.looping := 0;
							(mix_channel.item + l_i).item.fading := {PC_MIX_FADING}.mix_no_fading;
							(mix_channel.item + l_i).item.expire := (0).to_natural_32
							mix_channel_done_playing (l_i)
						else
							l_pos := -1 + 6 -- goto _L
						end
				when 5 then -- if block done, skip else block:
						l_pos := -1 + 7
				when 6 then -- else: | _L:
						if (mix_channel.item + l_i).item.fading /= {NATURAL_32} 0 then
							l_ticks := l_sdl_ticks - (mix_channel.item + l_i).item.ticks_fade
							if l_ticks > (mix_channel.item + l_i).item.fade_length then
								mix_volume (l_i, (mix_channel.item + l_i).item.fade_volume_reset).do_nothing
								if (mix_channel.item + l_i).item.fading = {NATURAL_32} 1 then
									(mix_channel.item + l_i).item.playing := 0;
									(mix_channel.item + l_i).item.looping := 0;
									(mix_channel.item + l_i).item.expire := (0).to_natural_32
									mix_channel_done_playing (l_i)
								end
								(mix_channel.item + l_i).item.fading := {PC_MIX_FADING}.mix_no_fading
							elseif (mix_channel.item + l_i).item.fading = {NATURAL_32} 1 then
								mix_volume (l_i, ((((mix_channel.item + l_i).item.fade_volume).to_natural_32 * ((mix_channel.item + l_i).item.fade_length - l_ticks)) // (mix_channel.item + l_i).item.fade_length).to_integer_32).do_nothing
							else
								mix_volume (l_i, ((((mix_channel.item + l_i).item.fade_volume).to_natural_32 * l_ticks) // (mix_channel.item + l_i).item.fade_length).to_integer_32).do_nothing
							end
						end
				when 7 then
						if (mix_channel.item + l_i).item.playing > 0 then
							l_index := 0
							l_remaining := a_len
							from
							until
								(l_break)
							loop
								l_break := False
								if (mix_channel.item + l_i).item.playing > 0 then
									if not (l_index < a_len) then
										l_break := True
									end
								else
									l_break := True
								end
								if not (l_break) then
									l_remaining := a_len - l_index
									l_volume := ((mix_channel.item + l_i).item.volume * (((mix_channel.item + l_i).item.chunk).item.volume).to_integer_32) // 128
									l_mixable := (mix_channel.item + l_i).item.playing
									if l_mixable > l_remaining then
										l_mixable := l_remaining
									end
									l_mix_input := create {CE_NATURAL_8_POINTER}.make_cast ((mix_do_effects (l_i, create {CE_VOID_POINTER}.make_cast ((mix_channel.item + l_i).item.samples), l_mixable)))
									sdl_mix_audio (a_stream + l_index, l_mix_input, l_mixable.to_natural_32, l_volume)
									if not l_mix_input.is_equal ((mix_channel.item + l_i).item.samples) then
										free (create {CE_VOID_POINTER}.make_cast (l_mix_input))
									end
									(mix_channel.item + l_i).item.samples := (mix_channel.item + l_i).item.samples + l_mixable;
									(mix_channel.item + l_i).item.playing := (mix_channel.item + l_i).item.playing - l_mixable
									l_index := l_index + l_mixable
									if not ((mix_channel.item + l_i).item.playing).to_boolean then
										if not ((mix_channel.item + l_i).item.looping).to_boolean then
											mix_channel_done_playing (l_i)
										end
									end
								end
							end
							l_break := False
							from
							until
								(l_break)
							loop
								l_break := False
								if ((mix_channel.item + l_i).item.looping).to_boolean then
									if not (l_index < a_len) then
										l_break := True
									end
								else
									l_break := True
								end
								if not (l_break) then
									l_alen := (((mix_channel.item + l_i).item.chunk).item.alen).to_integer_32
									l_remaining := a_len - l_index
									if l_remaining > l_alen then
										l_remaining := l_alen
									end
									l_mix_input := create {CE_NATURAL_8_POINTER}.make_cast ((mix_do_effects (l_i, create {CE_VOID_POINTER}.make_cast (((mix_channel.item + l_i).item.chunk).item.abuf), l_remaining)))
									sdl_mix_audio (a_stream + l_index, l_mix_input, l_remaining.to_natural_32, l_volume)
									if not l_mix_input.is_equal (((mix_channel.item + l_i).item.chunk).item.abuf) then
										free (create {CE_VOID_POINTER}.make_cast (l_mix_input))
									end
									(mix_channel.item + l_i).item.looping := ((mix_channel.item + l_i).item.looping) - 1;
									(mix_channel.item + l_i).item.samples := ((mix_channel.item + l_i).item.chunk).item.abuf + l_remaining;
									(mix_channel.item + l_i).item.playing := (((mix_channel.item + l_i).item.chunk).item.alen - l_remaining.to_natural_32).to_integer_32
									l_index := l_index + l_remaining
								end
							end
							l_break := False
							if not ((mix_channel.item + l_i).item.playing).to_boolean then
								if ((mix_channel.item + l_i).item.looping).to_boolean then
									(mix_channel.item + l_i).item.looping := ((mix_channel.item + l_i).item.looping) - 1;
									(mix_channel.item + l_i).item.samples := ((mix_channel.item + l_i).item.chunk).item.abuf;
									(mix_channel.item + l_i).item.playing := (((mix_channel.item + l_i).item.chunk).item.alen).to_integer_32
								end
							end
						end
				when 8 then -- if done
						l_i := l_i + 1
						l_pos := -1 + 1 -- back to loop condition
				when 9 then -- loop done
					mix_do_effects (-2, create {CE_VOID_POINTER}.make_cast (a_stream), a_len).do_nothing
					if (not mix_postmix.item.is_void) then
						mix_postmix.item.call ([mix_postmix_data.item, a_stream, a_len])
					end
				when 10 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	mix_open_audio (a_frequency: INTEGER_32; a_format: NATURAL_16; a_nchannels, a_chunksize: INTEGER_32): INTEGER_32
		local
			l_i: INTEGER_32
			l_desired: CE_ARRAY [SD_SDL_AUDIOSPEC]
			l_return: BOOLEAN
		do
			create {CE_CLASS_ARRAY [SD_SDL_AUDIOSPEC]} l_desired.make (<<1>>)

			if audio_opened.item.to_boolean then
				if a_format.to_integer_32 = (mixer.item.item.format).to_integer_32 then
					if a_nchannels = (mixer.item.item.channels).to_integer_32 then
						audio_opened.item := audio_opened.item + 1
						Result := 0
						l_return := True
					end
				end
				if not l_return then
					from
					until
						not (audio_opened.item.to_boolean)
					loop
						mix_close_audio
					end
				end
			end
			if not l_return then
				l_desired.item.freq := a_frequency
				l_desired.item.format := a_format
				l_desired.item.channels := a_nchannels.to_natural_8
				l_desired.item.samples := a_chunksize.to_natural_16
				l_desired.item.callback := mix_channels_address
				l_desired.item.userdata := create {CE_VOID_POINTER}.make_shared (default_pointer)
				if (sdl_open_audio (l_desired, mixer.item)) < 0 then
					Result := (-1)
					l_return := True
				end
			end
			if not l_return then
				if ((create {P_MUSIC_STATIC}).open_music (mixer.item)) < 0 then
					sdl_close_audio
					Result := (-1)
					l_return := True
				end
			end
			if not l_return then
				num_channels.item := 8
				mix_channel.item := create {CE_CLASS_POINTER [PD_MIX_CHANNEL]}.make_cast ((malloc (num_channels.item.to_natural_32 * (create {PD_MIX_CHANNEL}.make).structure_size.to_natural_32)))
				l_i := 0
				from
				until
					not (l_i < num_channels.item)
				loop
					(mix_channel.item + l_i).item.chunk := create {CE_CLASS_POINTER [PD_MIX_CHUNK]}.make_shared (default_pointer);
					(mix_channel.item + l_i).item.playing := 0;
					(mix_channel.item + l_i).item.looping := 0;
					(mix_channel.item + l_i).item.volume := 128;
					(mix_channel.item + l_i).item.fade_volume := 128;
					(mix_channel.item + l_i).item.fade_volume_reset := 128;
					(mix_channel.item + l_i).item.fading := {PC_MIX_FADING}.mix_no_fading;
					(mix_channel.item + l_i).item.tag := -1;
					(mix_channel.item + l_i).item.expire := (0).to_natural_32;
					(mix_channel.item + l_i).item.effects := create {CE_CLASS_POINTER [PD_MIX_EFFECTINFO]}.make_shared (default_pointer);
					(mix_channel.item + l_i).item.paused := 0
					l_i := l_i + 1
				end
				(create {P_MUSIC_STATIC}).mix_volume_music (128).do_nothing;
				(create {P_EFFECTS_INTERNAL_STATIC}).mix_init_effects
				add_chunk_decoder (ce_string ("WAVE"))
				add_chunk_decoder (ce_string ("AIFF"))
				add_chunk_decoder (ce_string ("VOC"))
				audio_opened.item := 1
				sdl_pause_audio (0)
				Result := 0
			end
		end

	mix_allocate_channels (a_numchans: INTEGER_32): INTEGER_32
		local
			l_i: INTEGER_32
			l_i_0: INTEGER_32
			l_return: BOOLEAN
		do
			if a_numchans < 0 then
				Result := num_channels.item
				l_return := True
			elseif a_numchans = num_channels.item then
				Result := num_channels.item
				l_return := True
			end
			if not l_return then
				if a_numchans < num_channels.item then
					l_i := a_numchans
					from
					until
						not (l_i < num_channels.item)
					loop
						mix_unregister_all_effects (l_i).do_nothing
						mix_halt_channel (l_i).do_nothing
						l_i := l_i + 1
					end
				end
				sdl_lock_audio
				mix_channel.item := create {CE_CLASS_POINTER [PD_MIX_CHANNEL]}.make_cast ((realloc (create {CE_VOID_POINTER}.make_cast (mix_channel.item), a_numchans.to_natural_32 * (create {PD_MIX_CHANNEL}.make).structure_size.to_natural_32)))
				if a_numchans > num_channels.item then
					l_i_0 := num_channels.item
					from
					until
						not (l_i_0 < a_numchans)
					loop
						(mix_channel.item + l_i_0).item.chunk := create {CE_CLASS_POINTER [PD_MIX_CHUNK]}.make_shared (default_pointer);
						(mix_channel.item + l_i_0).item.playing := 0;
						(mix_channel.item + l_i_0).item.looping := 0;
						(mix_channel.item + l_i_0).item.volume := 128;
						(mix_channel.item + l_i_0).item.fade_volume := 128;
						(mix_channel.item + l_i_0).item.fade_volume_reset := 128;
						(mix_channel.item + l_i_0).item.fading := {PC_MIX_FADING}.mix_no_fading;
						(mix_channel.item + l_i_0).item.tag := -1;
						(mix_channel.item + l_i_0).item.expire := (0).to_natural_32;
						(mix_channel.item + l_i_0).item.effects := create {CE_CLASS_POINTER [PD_MIX_EFFECTINFO]}.make_shared (default_pointer);
						(mix_channel.item + l_i_0).item.paused := 0
						l_i_0 := l_i_0 + 1
					end
				end
				num_channels.item := a_numchans
				sdl_unlock_audio
				Result := num_channels.item
			end
		end

	mix_query_spec (a_frequency: CE_POINTER [INTEGER_32]; a_format: CE_POINTER [NATURAL_16]; a_channels: CE_POINTER [INTEGER_32]): INTEGER_32
		do
			if audio_opened.item.to_boolean then
				if (not a_frequency.is_void) then
					a_frequency.item := mixer.item.item.freq
				end
				if (not a_format.is_void) then
					a_format.item := mixer.item.item.format
				end
				if (not a_channels.is_void) then
					a_channels.item := (mixer.item.item.channels).to_integer_32
				end
			end
			Result := audio_opened.item
		end

	mix_load_wav_rw (a_src: CE_POINTER [SD_SDL_RWOPS]; a_freesrc: INTEGER_32): CE_POINTER [PD_MIX_CHUNK]
		local
			l_magic: NATURAL_32
			l_chunk: CE_POINTER [PD_MIX_CHUNK]
			l_wavespec: CE_ARRAY [SD_SDL_AUDIOSPEC]
			l_loaded: CE_POINTER [SD_SDL_AUDIOSPEC]
			l_wavecvt: CE_ARRAY [SD_SDL_AUDIOCVT]
			l_samplesize: INTEGER_32
			l_return: BOOLEAN
			l_pos: INTEGER_32
		do
			create {CE_CLASS_ARRAY [SD_SDL_AUDIOSPEC]} l_wavespec.make (<<1>>)
			create {CE_CLASS_POINTER [SD_SDL_AUDIOSPEC]} l_loaded.make
			create {CE_CLASS_ARRAY [SD_SDL_AUDIOCVT]} l_wavecvt.make (<<1>>)

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					if a_src.is_void then
						sdl_set_error ([ce_string ("Mix_LoadWAV_RW with NULL src")])
						Result := (create {CE_CLASS_POINTER [PD_MIX_CHUNK]}.make_shared (default_pointer))
						l_return := True
					end
					if not l_return and then (not audio_opened.item.to_boolean) then
						sdl_set_error ([ce_string ("Audio device hasn%'t been opened")])
						if a_freesrc.to_boolean then
							if (not a_src.is_void) then
								(a_src.item.close).call ([a_src])
							end
						end
						Result := (create {CE_CLASS_POINTER [PD_MIX_CHUNK]}.make_shared (default_pointer))
						l_return := True
					end
					if not l_return then
						l_chunk := create {CE_CLASS_POINTER [PD_MIX_CHUNK]}.make_cast ((malloc ((create {PD_MIX_CHUNK}.make).structure_size.to_natural_32)))
						if l_chunk.is_void then
							sdl_set_error ([ce_string ("Out of memory")])
							if a_freesrc.to_boolean then
								(a_src.item.close).call ([a_src])
							end
							Result := (create {CE_CLASS_POINTER [PD_MIX_CHUNK]}.make_shared (default_pointer))
							l_return := True
						end
					end
					if l_return then
						l_pos := -1 + 6 -- skip if block
					end
				when 1 then -- if:
						l_magic := sdl_read_le32 (a_src);
						(a_src.item.seek).call ([a_src, -({PLATFORM}.Natural_32_bytes.to_natural_32).to_integer_32, 1])
						inspect l_magic -- switch 1 head
						when 1179011410 then l_pos := -1 + 2
						when 1163280727 then l_pos := -1 + 2
						when 1297239878 then l_pos := -1 + 3
						when 1634038339 then l_pos := -1 + 4
						else l_pos := -1 + 5
						end
				when 2 then -- switch 1, case 1179011410u | switch 1, case 1163280727u
						l_loaded := sdl_load_wav_rw (a_src, a_freesrc, l_wavespec, create {CE_POINTER_POINTER [NATURAL_8]}.make_shared (l_chunk.c_pointer + (create {PD_MIX_CHUNK}.make).offset_of ("abuf")), create {CE_NATURAL_32_POINTER}.make_shared (l_chunk.c_pointer + (create {PD_MIX_CHUNK}.make).offset_of ("alen")))
						l_pos := -1 + 6 -- switch 1; break
				when 3 then -- switch 1, case 1297239878u
						l_loaded := (create {P_LOAD_AIFF_STATIC}).mix_load_aiff_rw (a_src, a_freesrc, l_wavespec, create {CE_POINTER_POINTER [NATURAL_8]}.make_shared (l_chunk.c_pointer + (create {PD_MIX_CHUNK}.make).offset_of ("abuf")), create {CE_NATURAL_32_POINTER}.make_shared (l_chunk.c_pointer + (create {PD_MIX_CHUNK}.make).offset_of ("alen")))
						l_pos := -1 + 6 -- switch 1; break
				when 4 then -- switch 1, case 1634038339u
						l_loaded := (create {P_LOAD_VOC_STATIC}).mix_load_voc_rw (a_src, a_freesrc, l_wavespec, create {CE_POINTER_POINTER [NATURAL_8]}.make_shared (l_chunk.c_pointer + (create {PD_MIX_CHUNK}.make).offset_of ("abuf")), create {CE_NATURAL_32_POINTER}.make_shared (l_chunk.c_pointer + (create {PD_MIX_CHUNK}.make).offset_of ("alen")))
						l_pos := -1 + 6 -- switch 1; break
				when 5 then -- switch 1, default:
						sdl_set_error ([ce_string ("Unrecognized sound file type")])
						Result := (create {CE_CLASS_POINTER [PD_MIX_CHUNK]}.make_shared (default_pointer))
						l_return := True
				when 6 then -- switch 1, done: | if done
					if not l_return and then (l_loaded.is_void) then
						free (create {CE_VOID_POINTER}.make_cast (l_chunk))
						if a_freesrc.to_boolean then
							(a_src.item.close).call ([a_src])
						end
						Result := (create {CE_CLASS_POINTER [PD_MIX_CHUNK]}.make_shared (default_pointer))
						l_return := True
					end
					if l_return then
						l_pos := -1 + 15 -- skip if block
					end
				when 7 then -- if:
						if (l_wavespec.item.format).to_integer_32 = (mixer.item.item.format).to_integer_32 then
							l_pos := -1 + 10 -- skip if block
						end
				when 8 then -- if:
						l_pos := -1 + 14 -- goto _L
				when 9 then -- if block done, skip else block:
						l_pos := -1 + 15
				when 10 then -- else:
						if (l_wavespec.item.channels).to_integer_32 = (mixer.item.item.channels).to_integer_32 then
							l_pos := -1 + 13 -- skip if block
						end
				when 11 then -- if:
						l_pos := -1 + 14 -- goto _L
				when 12 then -- if block done, skip else block:
						l_pos := -1 + 15
				when 13 then -- else:
						if l_wavespec.item.freq = mixer.item.item.freq then
							l_pos := -1 + 15 -- skip if block
						end
				when 14 then -- if: | _L:
						if (sdl_build_audio_cvt (l_wavecvt, l_wavespec.item.format, l_wavespec.item.channels, l_wavespec.item.freq, mixer.item.item.format, mixer.item.item.channels, mixer.item.item.freq)) < 0 then
							free (create {CE_VOID_POINTER}.make_cast (l_chunk.item.abuf))
							free (create {CE_VOID_POINTER}.make_cast (l_chunk))
							Result := (create {CE_CLASS_POINTER [PD_MIX_CHUNK]}.make_shared (default_pointer))
							l_return := True
						end
						if not l_return then
							l_samplesize := ((((l_wavespec.item.format).to_integer_32).bit_and (255)) // 8) * (l_wavespec.item.channels).to_integer_32
							l_wavecvt.item.len := ((l_chunk.item.alen).bit_and ((((l_samplesize - 1)).bit_not).to_natural_32)).to_integer_32
							l_wavecvt.item.buf := create {CE_NATURAL_8_POINTER}.make_cast ((calloc ((1).to_natural_32, (l_wavecvt.item.len * l_wavecvt.item.len_mult).to_natural_32)))
							if (l_wavecvt.item.buf).is_void then
								sdl_set_error ([ce_string ("Out of memory")])
								free (create {CE_VOID_POINTER}.make_cast (l_chunk.item.abuf))
								free (create {CE_VOID_POINTER}.make_cast (l_chunk))
								Result := (create {CE_CLASS_POINTER [PD_MIX_CHUNK]}.make_shared (default_pointer))
								l_return := True
							end
						end
						if not l_return then
							memcpy (create {CE_VOID_POINTER}.make_cast (l_wavecvt.item.buf), create {CE_VOID_POINTER}.make_cast (l_chunk.item.abuf), l_chunk.item.alen).do_nothing
							free (create {CE_VOID_POINTER}.make_cast (l_chunk.item.abuf))
							if (sdl_convert_audio (l_wavecvt)) < 0 then
								free (create {CE_VOID_POINTER}.make_cast (l_wavecvt.item.buf))
								free (create {CE_VOID_POINTER}.make_cast (l_chunk))
								Result := (create {CE_CLASS_POINTER [PD_MIX_CHUNK]}.make_shared (default_pointer))
								l_return := True
							end
						end
						if not l_return then
							l_chunk.item.abuf := l_wavecvt.item.buf
							l_chunk.item.alen := (l_wavecvt.item.len_cvt).to_natural_32
						end
				when 15 then -- if done | if done | if done
					if not l_return then
						l_chunk.item.allocated := 1
						l_chunk.item.volume := (128).to_natural_8
						Result := l_chunk
					end
				when 16 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	mix_quick_load_wav (a_mem: CE_POINTER [NATURAL_8]): CE_POINTER [PD_MIX_CHUNK]
		local
			l_mem: CE_POINTER [NATURAL_8]
			l_chunk: CE_POINTER [PD_MIX_CHUNK]
			l_magic: CE_ARRAY [NATURAL_8]
			l_return, l_break: BOOLEAN
		do
			create {CE_NATURAL_8_ARRAY} l_magic.make (<<4>>)

			l_mem := a_mem
			if not audio_opened.item.to_boolean then
				sdl_set_error ([ce_string ("Audio device hasn%'t been opened")])
				Result := (create {CE_CLASS_POINTER [PD_MIX_CHUNK]}.make_shared (default_pointer))
				l_return := True
			end
			if not l_return then
				l_chunk := create {CE_CLASS_POINTER [PD_MIX_CHUNK]}.make_cast ((calloc ((1).to_natural_32, (create {PD_MIX_CHUNK}.make).structure_size.to_natural_32)))
				if l_chunk.is_void then
					sdl_set_error ([ce_string ("Out of memory")])
					Result := (create {CE_CLASS_POINTER [PD_MIX_CHUNK]}.make_shared (default_pointer))
					l_return := True
				end
			end
			if not l_return then
				l_chunk.item.allocated := 0
				l_mem := l_mem + (12)
				from
				until
					(l_break)
				loop
					l_break := False
					memcpy (create {CE_VOID_POINTER}.make_cast (l_magic), create {CE_VOID_POINTER}.make_cast (l_mem), (4).to_natural_32).do_nothing
					l_mem := l_mem + (4)
					l_chunk.item.alen := (((((((((l_mem[3]).to_integer_32).bit_shift_left (24))).bit_or ((((l_mem[2]).to_integer_32).bit_shift_left (16))))).bit_or ((((l_mem[1]).to_integer_32).bit_shift_left (8))))).bit_or ((l_mem[0]).to_integer_32)).to_natural_32
					l_mem := l_mem + (4)
					l_chunk.item.abuf := l_mem
					l_mem := l_mem + (l_chunk.item.alen).to_integer_32
					if (memcmp (create {CE_VOID_POINTER}.make_cast (l_magic), create {CE_VOID_POINTER}.make_cast (ce_string ("data")), (4).to_natural_32)) = 0 then
						l_break := True
					end
				end
				l_break := False
				l_chunk.item.volume := (128).to_natural_8
				Result := l_chunk
			end
		end

	mix_quick_load_raw (a_mem: CE_POINTER [NATURAL_8]; a_len: NATURAL_32): CE_POINTER [PD_MIX_CHUNK]
		local
			l_chunk: CE_POINTER [PD_MIX_CHUNK]
			l_return: BOOLEAN
		do
			if not audio_opened.item.to_boolean then
				sdl_set_error ([ce_string ("Audio device hasn%'t been opened")])
				Result := (create {CE_CLASS_POINTER [PD_MIX_CHUNK]}.make_shared (default_pointer))
				l_return := True
			end
			if not l_return then
				l_chunk := create {CE_CLASS_POINTER [PD_MIX_CHUNK]}.make_cast ((malloc ((create {PD_MIX_CHUNK}.make).structure_size.to_natural_32)))
				if l_chunk.is_void then
					sdl_set_error ([ce_string ("Out of memory")])
					Result := (create {CE_CLASS_POINTER [PD_MIX_CHUNK]}.make_shared (default_pointer))
					l_return := True
				end
			end
			if not l_return then
				l_chunk.item.allocated := 0
				l_chunk.item.alen := a_len
				l_chunk.item.abuf := a_mem
				l_chunk.item.volume := (128).to_natural_8
				Result := l_chunk
			end
		end

	mix_free_chunk (a_chunk: CE_POINTER [PD_MIX_CHUNK])
		local
			l_i: INTEGER_32
		do
			if (not a_chunk.is_void) then
				sdl_lock_audio
				if (not mix_channel.item.is_void) then
					l_i := 0
					from
					until
						not (l_i < num_channels.item)
					loop
						if a_chunk.is_equal ((mix_channel.item + l_i).item.chunk) then
							(mix_channel.item + l_i).item.playing := 0;
							(mix_channel.item + l_i).item.looping := 0
						end
						l_i := l_i + 1
					end
				end
				sdl_unlock_audio
				if (a_chunk.item.allocated).to_boolean then
					free (create {CE_VOID_POINTER}.make_cast (a_chunk.item.abuf))
				end
				free (create {CE_VOID_POINTER}.make_cast (a_chunk))
			end
		end

	mix_set_post_mix (a_mix_func: CE_ROUTINE; a_arg: CE_POINTER [ANY])
		do
			sdl_lock_audio
			mix_postmix_data.item := a_arg
			mix_postmix.item := a_mix_func
			sdl_unlock_audio
		end

	mix_hook_music (a_mix_func: CE_ROUTINE; a_arg: CE_POINTER [ANY])
		do
			sdl_lock_audio
			if not a_mix_func.is_void then
				music_data.item := a_arg
				mix_music.item := a_mix_func
			else
				music_data.item := create {CE_VOID_POINTER}.make_shared (default_pointer)
				mix_music.item := (create {P_MUSIC_STATIC}).music_mixer_address
			end
			sdl_unlock_audio
		end

	mix_get_music_hook_data: CE_POINTER [ANY]
		do
			Result := music_data.item
		end

	mix_channel_finished (a_channel_finished: CE_ROUTINE)
		do
			sdl_lock_audio
			channel_done_callback.item := a_channel_finished
			sdl_unlock_audio
		end

	mix_reserve_channels (a_num: INTEGER_32): INTEGER_32
		local
			l_num: INTEGER_32
		do
			l_num := a_num
			if l_num > num_channels.item then
				l_num := num_channels.item
			end
			reserved_channels.item := l_num
			Result := l_num
		end

	checkchunkintegral (a_chunk: CE_POINTER [PD_MIX_CHUNK]): INTEGER_32
		local
			l_frame_width: INTEGER_32
		do
			l_frame_width := 1
			if (((mixer.item.item.format).to_integer_32).bit_and (255)) = 16 then
				l_frame_width := 2
			end
			l_frame_width := l_frame_width * (mixer.item.item.channels).to_integer_32
			from
			until
				not ((a_chunk.item.alen \\ l_frame_width.to_natural_32).to_boolean)
			loop
				a_chunk.item.alen := (a_chunk.item.alen) - 1
			end
			Result := (a_chunk.item.alen).to_integer_32
		end

	mix_play_channel_timed (a_which: INTEGER_32; a_chunk: CE_POINTER [PD_MIX_CHUNK]; a_loops, a_ticks: INTEGER_32): INTEGER_32
		local
			l_which: INTEGER_32
			l_i: INTEGER_32
			l_sdl_ticks: NATURAL_32
			l_return, l_break: BOOLEAN
		do
			l_which := a_which
			if a_chunk.is_void then
				sdl_set_error ([ce_string ("Tried to play a NULL chunk")])
				Result := (-1)
				l_return := True
			end
			if not l_return then
				if not (checkchunkintegral (a_chunk)).to_boolean then
					sdl_set_error ([ce_string ("Tried to play a chunk with a bad frame")])
					Result := (-1)
					l_return := True
				end
			end
			if not l_return then
				sdl_lock_audio
				if l_which = -1 then
					l_i := reserved_channels.item
					from
					until
						not (not (l_break) and then (l_i < num_channels.item))
					loop
						l_break := False
						if (mix_channel.item + l_i).item.playing <= 0 then
							l_break := True
						end
						if not (l_break) then
							l_i := l_i + 1
						end
					end
					l_break := False
					if l_i = num_channels.item then
						sdl_set_error ([ce_string ("No free channels available")])
						l_which := -1
					else
						l_which := l_i
					end
				end
				if l_which >= 0 then
					if l_which < num_channels.item then
						l_sdl_ticks := sdl_get_ticks
						if (mix_playing (l_which)).to_boolean then
							mix_channel_done_playing (l_which)
						end
						(mix_channel.item + l_which).item.samples := a_chunk.item.abuf;
						(mix_channel.item + l_which).item.playing := (a_chunk.item.alen).to_integer_32;
						(mix_channel.item + l_which).item.looping := a_loops;
						(mix_channel.item + l_which).item.chunk := a_chunk;
						(mix_channel.item + l_which).item.paused := 0;
						(mix_channel.item + l_which).item.fading := {PC_MIX_FADING}.mix_no_fading;
						(mix_channel.item + l_which).item.start_time := l_sdl_ticks
						if a_ticks > 0 then
							(mix_channel.item + l_which).item.expire := l_sdl_ticks + a_ticks.to_natural_32
						else
							(mix_channel.item + l_which).item.expire := (0).to_natural_32
						end
					end
				end
				sdl_unlock_audio
				Result := l_which
			end
		end

	mix_expire_channel (a_which, a_ticks: INTEGER_32): INTEGER_32
		local
			l_status: INTEGER_32
			l_i: INTEGER_32
		do
			l_status := 0
			if a_which = -1 then
				l_i := 0
				from
				until
					not (l_i < num_channels.item)
				loop
					l_status := l_status + (mix_expire_channel (l_i, a_ticks))
					l_i := l_i + 1
				end
			elseif a_which < num_channels.item then
				sdl_lock_audio
				if a_ticks > 0 then
					(mix_channel.item + a_which).item.expire := sdl_get_ticks + a_ticks.to_natural_32
				else
					(mix_channel.item + a_which).item.expire := (0).to_natural_32
				end
				sdl_unlock_audio
				l_status := l_status + 1
			end
			Result := l_status
		end

	mix_fade_in_channel_timed (a_which: INTEGER_32; a_chunk: CE_POINTER [PD_MIX_CHUNK]; a_loops, a_ms, a_ticks: INTEGER_32): INTEGER_32
		local
			l_which: INTEGER_32
			l_i: INTEGER_32
			l_sdl_ticks: NATURAL_32
			l_tmp_2: NATURAL_32
			l_return, l_break: BOOLEAN
		do
			l_which := a_which
			if a_chunk.is_void then
				Result := (-1)
				l_return := True
			end
			if not l_return then
				if not (checkchunkintegral (a_chunk)).to_boolean then
					sdl_set_error ([ce_string ("Tried to play a chunk with a bad frame")])
					Result := (-1)
					l_return := True
				end
			end
			if not l_return then
				sdl_lock_audio
				if l_which = -1 then
					l_i := reserved_channels.item
					from
					until
						not (not (l_break) and then (l_i < num_channels.item))
					loop
						l_break := False
						if (mix_channel.item + l_i).item.playing <= 0 then
							l_break := True
						end
						if not (l_break) then
							l_i := l_i + 1
						end
					end
					l_break := False
					if l_i = num_channels.item then
						l_which := -1
					else
						l_which := l_i
					end
				end
				if l_which >= 0 then
					if l_which < num_channels.item then
						l_sdl_ticks := sdl_get_ticks
						if (mix_playing (l_which)).to_boolean then
							mix_channel_done_playing (l_which)
						end
						(mix_channel.item + l_which).item.samples := a_chunk.item.abuf;
						(mix_channel.item + l_which).item.playing := (a_chunk.item.alen).to_integer_32;
						(mix_channel.item + l_which).item.looping := a_loops;
						(mix_channel.item + l_which).item.chunk := a_chunk;
						(mix_channel.item + l_which).item.paused := 0;
						(mix_channel.item + l_which).item.fading := {PC_MIX_FADING}.mix_fading_in;
						(mix_channel.item + l_which).item.fade_volume := (mix_channel.item + l_which).item.volume;
						(mix_channel.item + l_which).item.fade_volume_reset := (mix_channel.item + l_which).item.volume;
						(mix_channel.item + l_which).item.volume := 0;
						(mix_channel.item + l_which).item.fade_length := a_ms.to_natural_32
						l_tmp_2 := l_sdl_ticks;
						(mix_channel.item + l_which).item.ticks_fade := l_tmp_2;
						(mix_channel.item + l_which).item.start_time := l_tmp_2
						if a_ticks > 0 then
							(mix_channel.item + l_which).item.expire := l_sdl_ticks + a_ticks.to_natural_32
						else
							(mix_channel.item + l_which).item.expire := (0).to_natural_32
						end
					end
				end
				sdl_unlock_audio
				Result := l_which
			end
		end

	mix_volume (a_which, a_volume: INTEGER_32): INTEGER_32
		local
			l_volume: INTEGER_32
			l_i: INTEGER_32
			l_prev_volume: INTEGER_32
		do
			l_volume := a_volume
			l_prev_volume := 0
			if a_which = -1 then
				l_i := 0
				from
				until
					not (l_i < num_channels.item)
				loop
					l_prev_volume := l_prev_volume + (mix_volume (l_i, l_volume))
					l_i := l_i + 1
				end
				l_prev_volume := l_prev_volume // num_channels.item
			elseif a_which < num_channels.item then
				l_prev_volume := (mix_channel.item + a_which).item.volume
				if l_volume >= 0 then
					if l_volume > 128 then
						l_volume := 128
					end
					(mix_channel.item + a_which).item.volume := l_volume
				end
			end
			Result := l_prev_volume
		end

	mix_volume_chunk (a_chunk: CE_POINTER [PD_MIX_CHUNK]; a_volume: INTEGER_32): INTEGER_32
		local
			l_volume: INTEGER_32
			l_prev_volume: INTEGER_32
		do
			l_volume := a_volume
			l_prev_volume := (a_chunk.item.volume).to_integer_32
			if l_volume >= 0 then
				if l_volume > 128 then
					l_volume := 128
				end
				a_chunk.item.volume := l_volume.to_natural_8
			end
			Result := l_prev_volume
		end

	mix_halt_channel (a_which: INTEGER_32): INTEGER_32
		local
			l_i: INTEGER_32
		do
			if a_which = -1 then
				l_i := 0
				from
				until
					not (l_i < num_channels.item)
				loop
					mix_halt_channel (l_i).do_nothing
					l_i := l_i + 1
				end
			elseif a_which < num_channels.item then
				sdl_lock_audio
				if ((mix_channel.item + a_which).item.playing).to_boolean then
					mix_channel_done_playing (a_which);
					(mix_channel.item + a_which).item.playing := 0;
					(mix_channel.item + a_which).item.looping := 0
				end
				(mix_channel.item + a_which).item.expire := (0).to_natural_32
				if (mix_channel.item + a_which).item.fading /= {NATURAL_32} 0 then
					(mix_channel.item + a_which).item.volume := (mix_channel.item + a_which).item.fade_volume_reset
				end
				(mix_channel.item + a_which).item.fading := {PC_MIX_FADING}.mix_no_fading
				sdl_unlock_audio
			end
			Result := 0
		end

	mix_halt_group (a_tag: INTEGER_32): INTEGER_32
		local
			l_i: INTEGER_32
		do
			l_i := 0
			from
			until
				not (l_i < num_channels.item)
			loop
				if (mix_channel.item + l_i).item.tag = a_tag then
					mix_halt_channel (l_i).do_nothing
				end
				l_i := l_i + 1
			end
			Result := 0
		end

	mix_fade_out_channel (a_which, a_ms: INTEGER_32): INTEGER_32
		local
			l_status: INTEGER_32
			l_i: INTEGER_32
		do
			l_status := 0
			if audio_opened.item.to_boolean then
				if a_which = -1 then
					l_i := 0
					from
					until
						not (l_i < num_channels.item)
					loop
						l_status := l_status + (mix_fade_out_channel (l_i, a_ms))
						l_i := l_i + 1
					end
				elseif a_which < num_channels.item then
					sdl_lock_audio
					if ((mix_channel.item + a_which).item.playing).to_boolean then
						if (mix_channel.item + a_which).item.volume > 0 then
							if (mix_channel.item + a_which).item.fading /= {NATURAL_32} 1 then
								(mix_channel.item + a_which).item.fade_volume := (mix_channel.item + a_which).item.volume;
								(mix_channel.item + a_which).item.fading := {PC_MIX_FADING}.mix_fading_out;
								(mix_channel.item + a_which).item.fade_length := a_ms.to_natural_32;
								(mix_channel.item + a_which).item.ticks_fade := sdl_get_ticks
								if (mix_channel.item + a_which).item.fading = {NATURAL_32} 0 then
									(mix_channel.item + a_which).item.fade_volume_reset := (mix_channel.item + a_which).item.volume
								end
								l_status := l_status + 1
							end
						end
					end
					sdl_unlock_audio
				end
			end
			Result := l_status
		end

	mix_fade_out_group (a_tag, a_ms: INTEGER_32): INTEGER_32
		local
			l_i: INTEGER_32
			l_status: INTEGER_32
		do
			l_status := 0
			l_i := 0
			from
			until
				not (l_i < num_channels.item)
			loop
				if (mix_channel.item + l_i).item.tag = a_tag then
					l_status := l_status + (mix_fade_out_channel (l_i, a_ms))
				end
				l_i := l_i + 1
			end
			Result := l_status
		end

	mix_fading_channel (a_which: INTEGER_32): NATURAL_32
		local
			l_return: BOOLEAN
		do
			if a_which < 0 then
				Result := ({PC_MIX_FADING}.mix_no_fading)
				l_return := True
			elseif a_which >= num_channels.item then
				Result := ({PC_MIX_FADING}.mix_no_fading)
				l_return := True
			end
			if not l_return then
				Result := ((mix_channel.item + a_which).item.fading)
			end
		end

	mix_playing (a_which: INTEGER_32): INTEGER_32
		local
			l_status: INTEGER_32
			l_i: INTEGER_32
		do
			l_status := 0
			if a_which = -1 then
				l_i := 0
				from
				until
					not (l_i < num_channels.item)
				loop
					if (mix_channel.item + l_i).item.playing > 0 then
						l_status := l_status + 1
					elseif (mix_channel.item + l_i).item.looping > 0 then
						l_status := l_status + 1
					end
					l_i := l_i + 1
				end
			elseif a_which < num_channels.item then
				if (mix_channel.item + a_which).item.playing > 0 then
					l_status := l_status + 1
				elseif (mix_channel.item + a_which).item.looping > 0 then
					l_status := l_status + 1
				end
			end
			Result := l_status
		end

	mix_get_chunk (a_channel: INTEGER_32): CE_POINTER [PD_MIX_CHUNK]
		local
			l_retval: CE_POINTER [PD_MIX_CHUNK]
		do
			l_retval := create {CE_CLASS_POINTER [PD_MIX_CHUNK]}.make_shared (default_pointer)
			if a_channel >= 0 then
				if a_channel < num_channels.item then
					l_retval := (mix_channel.item + a_channel).item.chunk
				end
			end
			Result := l_retval
		end

	mix_close_audio
		local
			l_i: INTEGER_32
		do
			if audio_opened.item.to_boolean then
				if audio_opened.item = 1 then
					l_i := 0
					from
					until
						not (l_i < num_channels.item)
					loop
						mix_unregister_all_effects (l_i).do_nothing
						l_i := l_i + 1
					end
					mix_unregister_all_effects (-2).do_nothing;
					(create {P_MUSIC_STATIC}).close_music
					mix_halt_channel (-1).do_nothing;
					(create {P_EFFECTS_INTERNAL_STATIC}).mix_deinit_effects
					sdl_close_audio
					free (create {CE_VOID_POINTER}.make_cast (mix_channel.item))
					mix_channel.item := create {CE_CLASS_POINTER [PD_MIX_CHANNEL]}.make_shared (default_pointer)
					free (create {CE_VOID_POINTER}.make_cast (chunk_decoders.item))
					chunk_decoders.item := create {CE_POINTER_POINTER [INTEGER_8]}.make_shared (default_pointer)
					num_decoders.item := 0
				end
				audio_opened.item := audio_opened.item - 1
			end
		end

	mix_pause (a_which: INTEGER_32)
		local
			l_sdl_ticks: NATURAL_32
			l_i: INTEGER_32
		do
			l_sdl_ticks := sdl_get_ticks
			if a_which = -1 then
				l_i := 0
				from
				until
					not (l_i < num_channels.item)
				loop
					if (mix_channel.item + l_i).item.playing > 0 then
						(mix_channel.item + l_i).item.paused := l_sdl_ticks.to_integer_32
					end
					l_i := l_i + 1
				end
			elseif a_which < num_channels.item then
				if (mix_channel.item + a_which).item.playing > 0 then
					(mix_channel.item + a_which).item.paused := l_sdl_ticks.to_integer_32
				end
			end
		end

	mix_resume (a_which: INTEGER_32)
		local
			l_sdl_ticks: NATURAL_32
			l_i: INTEGER_32
		do
			l_sdl_ticks := sdl_get_ticks
			sdl_lock_audio
			if a_which = -1 then
				l_i := 0
				from
				until
					not (l_i < num_channels.item)
				loop
					if (mix_channel.item + l_i).item.playing > 0 then
						if (mix_channel.item + l_i).item.expire > {NATURAL_32} 0 then
							(mix_channel.item + l_i).item.expire := (mix_channel.item + l_i).item.expire + (l_sdl_ticks - ((mix_channel.item + l_i).item.paused).to_natural_32)
						end
						(mix_channel.item + l_i).item.paused := 0
					end
					l_i := l_i + 1
				end
			elseif a_which < num_channels.item then
				if (mix_channel.item + a_which).item.playing > 0 then
					if (mix_channel.item + a_which).item.expire > {NATURAL_32} 0 then
						(mix_channel.item + a_which).item.expire := (mix_channel.item + a_which).item.expire + (l_sdl_ticks - ((mix_channel.item + a_which).item.paused).to_natural_32)
					end
					(mix_channel.item + a_which).item.paused := 0
				end
			end
			sdl_unlock_audio
		end

	mix_paused (a_which: INTEGER_32): INTEGER_32
		local
			l_status: INTEGER_32
			l_i: INTEGER_32
		do
			if a_which < 0 then
				l_status := 0
				l_i := 0
				from
				until
					not (l_i < num_channels.item)
				loop
					if ((mix_channel.item + l_i).item.paused).to_boolean then
						l_status := l_status + 1
					end
					l_i := l_i + 1
				end
				Result := l_status
			elseif a_which < num_channels.item then
				Result := ((mix_channel.item + a_which).item.paused /= 0).to_integer
			else
				Result := 0
			end
		end

	mix_group_channel (a_which, a_tag: INTEGER_32): INTEGER_32
		local
			l_return: BOOLEAN
		do
			if a_which < 0 then
				Result := 0
				l_return := True
			elseif a_which > num_channels.item then
				Result := 0
				l_return := True
			end
			if not l_return then
				sdl_lock_audio;
				(mix_channel.item + a_which).item.tag := a_tag
				sdl_unlock_audio
				Result := 1
			end
		end

	mix_group_channels (a_from, a_to, a_tag: INTEGER_32): INTEGER_32
		local
			l_from: INTEGER_32
			l_status: INTEGER_32
		do
			l_from := a_from
			l_status := 0
			from
			until
				not (l_from <= a_to)
			loop
				l_status := l_status + (mix_group_channel (l_from, a_tag))
				l_from := l_from + 1
			end
			Result := l_status
		end

	mix_group_available (a_tag: INTEGER_32): INTEGER_32
		local
			l_i: INTEGER_32
			l_return: BOOLEAN
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_i := 0
				when 1 then -- loop condition:
					if not (not (l_return) and then (l_i < num_channels.item)) then
						l_pos := -1 + 8 -- skip loop
					end
				when 2 then -- loop body:
						if a_tag /= -1 then
							l_pos := -1 + 5 -- skip if block
						end
				when 3 then -- if:
						l_pos := -1 + 6 -- goto _L
				when 4 then -- if block done, skip else block:
						l_pos := -1 + 7
				when 5 then -- else:
						if a_tag /= (mix_channel.item + l_i).item.tag then
							l_pos := -1 + 7 -- skip if block
						end
				when 6 then -- if: | _L:
						if (mix_channel.item + l_i).item.playing <= 0 then
							Result := l_i
							l_return := True
						end
				when 7 then -- if done
						if not l_return then
							l_i := l_i + 1
						end
						l_pos := -1 + 1 -- back to loop condition
				when 8 then -- loop done
					if not l_return then
						Result := (-1)
					end
				when 9 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	mix_group_count (a_tag: INTEGER_32): INTEGER_32
		local
			l_count: INTEGER_32
			l_i: INTEGER_32
		do
			l_count := 0
			l_i := 0
			from
			until
				not (l_i < num_channels.item)
			loop
				if (mix_channel.item + l_i).item.tag = a_tag then
					l_count := l_count + 1
				elseif a_tag = -1 then
					l_count := l_count + 1
				end
				l_i := l_i + 1
			end
			Result := l_count
		end

	mix_group_oldest (a_tag: INTEGER_32): INTEGER_32
		local
			l_chan: INTEGER_32
			l_mintime: NATURAL_32
			l_i: INTEGER_32
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_chan := -1
					l_mintime := sdl_get_ticks
					l_i := 0
				when 1 then -- loop condition:
					if not (l_i < num_channels.item) then
						l_pos := -1 + 8 -- skip loop
					end
				when 2 then -- loop body:
						if (mix_channel.item + l_i).item.tag /= a_tag then
							l_pos := -1 + 5 -- skip if block
						end
				when 3 then -- if:
						l_pos := -1 + 6 -- goto _L
				when 4 then -- if block done, skip else block:
						l_pos := -1 + 7
				when 5 then -- else:
						if a_tag /= -1 then
							l_pos := -1 + 7 -- skip if block
						end
				when 6 then -- if: | _L:
						if (mix_channel.item + l_i).item.playing > 0 then
							if (mix_channel.item + l_i).item.start_time <= l_mintime then
								l_mintime := (mix_channel.item + l_i).item.start_time
								l_chan := l_i
							end
						end
				when 7 then -- if done
						l_i := l_i + 1
						l_pos := -1 + 1 -- back to loop condition
				when 8 then -- loop done
					Result := l_chan
				when 9 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	mix_group_newer (a_tag: INTEGER_32): INTEGER_32
		local
			l_chan: INTEGER_32
			l_maxtime: NATURAL_32
			l_i: INTEGER_32
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_chan := -1
					l_maxtime := (0).to_natural_32
					l_i := 0
				when 1 then -- loop condition:
					if not (l_i < num_channels.item) then
						l_pos := -1 + 8 -- skip loop
					end
				when 2 then -- loop body:
						if (mix_channel.item + l_i).item.tag /= a_tag then
							l_pos := -1 + 5 -- skip if block
						end
				when 3 then -- if:
						l_pos := -1 + 6 -- goto _L
				when 4 then -- if block done, skip else block:
						l_pos := -1 + 7
				when 5 then -- else:
						if a_tag /= -1 then
							l_pos := -1 + 7 -- skip if block
						end
				when 6 then -- if: | _L:
						if (mix_channel.item + l_i).item.playing > 0 then
							if (mix_channel.item + l_i).item.start_time >= l_maxtime then
								l_maxtime := (mix_channel.item + l_i).item.start_time
								l_chan := l_i
							end
						end
				when 7 then -- if done
						l_i := l_i + 1
						l_pos := -1 + 1 -- back to loop condition
				when 8 then -- loop done
					Result := l_chan
				when 9 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	mix_register_effect (a_e: CE_POINTER [CE_POINTER [PD_MIX_EFFECTINFO]]; a_f, a_d: CE_ROUTINE; a_arg: CE_POINTER [ANY]): INTEGER_32
		local
			l_new_e: CE_POINTER [PD_MIX_EFFECTINFO]
			l_cur: CE_POINTER [PD_MIX_EFFECTINFO]
			l_return, l_break: BOOLEAN
		do
			if a_e.is_void then
				sdl_set_error ([ce_string ("Internal error")])
				Result := 0
				l_return := True
			end
			if not l_return and then (a_f.is_void) then
				sdl_set_error ([ce_string ("NULL effect callback")])
				Result := 0
				l_return := True
			end
			if not l_return then
				l_new_e := create {CE_CLASS_POINTER [PD_MIX_EFFECTINFO]}.make_cast ((malloc ((create {PD_MIX_EFFECTINFO}.make).structure_size.to_natural_32)))
				if l_new_e.is_void then
					sdl_set_error ([ce_string ("Out of memory")])
					Result := 0
					l_return := True
				end
			end
			if not l_return then
				l_new_e.item.callback := a_f
				l_new_e.item.done_callback := a_d
				l_new_e.item.udata := a_arg
				l_new_e.item.next := create {CE_CLASS_POINTER [PD_MIX_EFFECTINFO]}.make_shared (default_pointer)
				if (a_e.item).is_void then
					a_e.item := l_new_e
				else
					l_cur := a_e.item
					from
					until
						(l_break)
					loop
						l_break := False
						if (l_cur.item.next).is_void then
							l_cur.item.next := l_new_e
							l_break := True
						end
						if not (l_break) then
							l_cur := l_cur.item.next
						end
					end
					l_break := False
				end
				Result := 1
			end
		end

	mix_remove_effect (a_channel: INTEGER_32; a_e: CE_POINTER [CE_POINTER [PD_MIX_EFFECTINFO]]; a_f: CE_ROUTINE): INTEGER_32
		local
			l_cur: CE_POINTER [PD_MIX_EFFECTINFO]
			l_prev: CE_POINTER [PD_MIX_EFFECTINFO]
			l_next: CE_POINTER [PD_MIX_EFFECTINFO]
			l_return: BOOLEAN
		do
			l_prev := create {CE_CLASS_POINTER [PD_MIX_EFFECTINFO]}.make_shared (default_pointer)
			l_next := create {CE_CLASS_POINTER [PD_MIX_EFFECTINFO]}.make_shared (default_pointer)
			if a_e.is_void then
				sdl_set_error ([ce_string ("Internal error")])
				Result := 0
				l_return := True
			end
			if not l_return then
				l_cur := a_e.item
				from
				until
					not (not (l_return) and then (not l_cur.is_void))
				loop
					if (l_cur.item.callback).is_equal (a_f) then
						l_next := l_cur.item.next
						if not (l_cur.item.done_callback).is_void then
							(l_cur.item.done_callback).call ([a_channel, l_cur.item.udata])
						end
						free (create {CE_VOID_POINTER}.make_cast (l_cur))
						if l_prev.is_void then
							a_e.item := l_next
						else
							l_prev.item.next := l_next
						end
						Result := 1
						l_return := True
					end
					if not l_return then
						l_prev := l_cur
						l_cur := l_cur.item.next
					end
				end
			end
			if not l_return then
				sdl_set_error ([ce_string ("No such effect registered")])
				Result := 0
			end
		end

	mix_remove_all_effects (a_channel: INTEGER_32; a_e: CE_POINTER [CE_POINTER [PD_MIX_EFFECTINFO]]): INTEGER_32
		local
			l_cur: CE_POINTER [PD_MIX_EFFECTINFO]
			l_next: CE_POINTER [PD_MIX_EFFECTINFO]
			l_return: BOOLEAN
		do
			if a_e.is_void then
				sdl_set_error ([ce_string ("Internal error")])
				Result := 0
				l_return := True
			end
			if not l_return then
				l_cur := a_e.item
				from
				until
					l_cur.is_void
				loop
					l_next := l_cur.item.next
					if not (l_cur.item.done_callback).is_void then
						(l_cur.item.done_callback).call ([a_channel, l_cur.item.udata])
					end
					free (create {CE_VOID_POINTER}.make_cast (l_cur))
					l_cur := l_next
				end
				a_e.item := create {CE_CLASS_POINTER [PD_MIX_EFFECTINFO]}.make_shared (default_pointer)
				Result := 1
			end
		end

	mix_register_effect_locked (a_channel: INTEGER_32; a_f, a_d: CE_ROUTINE; a_arg: CE_POINTER [ANY]): INTEGER_32
		local
			l_e: CE_POINTER [CE_POINTER [PD_MIX_EFFECTINFO]]
			l_return: BOOLEAN
		do
			l_e := create {CE_POINTER_POINTER [PD_MIX_EFFECTINFO]}.make_shared (default_pointer)
			if a_channel = -2 then
				l_e := posteffects.item
			else
				if a_channel < 0 then
					sdl_set_error ([ce_string ("Invalid channel number")])
					Result := 0
					l_return := True
				elseif a_channel >= num_channels.item then
					sdl_set_error ([ce_string ("Invalid channel number")])
					Result := 0
					l_return := True
				end
				if not l_return then
					l_e := create {CE_POINTER_POINTER [PD_MIX_EFFECTINFO]}.make_shared ((mix_channel.item + a_channel).c_pointer + (create {PD_MIX_CHANNEL}.make).offset_of ("effects"))
				end
			end
			if not l_return then
				Result := (mix_register_effect (l_e, a_f, a_d, a_arg))
			end
		end

	mix_register_effect2 (a_channel: INTEGER_32; a_f, a_d: CE_ROUTINE; a_arg: CE_POINTER [ANY]): INTEGER_32
		local
			l_retval: INTEGER_32
		do
			sdl_lock_audio
			l_retval := mix_register_effect_locked (a_channel, a_f, a_d, a_arg)
			sdl_unlock_audio
			Result := l_retval
		end

	mix_unregister_effect_locked (a_channel: INTEGER_32; a_f: CE_ROUTINE): INTEGER_32
		local
			l_e: CE_POINTER [CE_POINTER [PD_MIX_EFFECTINFO]]
			l_return: BOOLEAN
		do
			l_e := create {CE_POINTER_POINTER [PD_MIX_EFFECTINFO]}.make_shared (default_pointer)
			if a_channel = -2 then
				l_e := posteffects.item
			else
				if a_channel < 0 then
					sdl_set_error ([ce_string ("Invalid channel number")])
					Result := 0
					l_return := True
				elseif a_channel >= num_channels.item then
					sdl_set_error ([ce_string ("Invalid channel number")])
					Result := 0
					l_return := True
				end
				if not l_return then
					l_e := create {CE_POINTER_POINTER [PD_MIX_EFFECTINFO]}.make_shared ((mix_channel.item + a_channel).c_pointer + (create {PD_MIX_CHANNEL}.make).offset_of ("effects"))
				end
			end
			if not l_return then
				Result := (mix_remove_effect (a_channel, l_e, a_f))
			end
		end

	mix_unregister_effect (a_channel: INTEGER_32; a_f: CE_ROUTINE): INTEGER_32
		local
			l_retval: INTEGER_32
		do
			sdl_lock_audio
			l_retval := mix_unregister_effect_locked (a_channel, a_f)
			sdl_unlock_audio
			Result := l_retval
		end

	mix_unregister_all_effects_locked (a_channel: INTEGER_32): INTEGER_32
		local
			l_e: CE_POINTER [CE_POINTER [PD_MIX_EFFECTINFO]]
			l_return: BOOLEAN
		do
			l_e := create {CE_POINTER_POINTER [PD_MIX_EFFECTINFO]}.make_shared (default_pointer)
			if a_channel = -2 then
				l_e := posteffects.item
			else
				if a_channel < 0 then
					sdl_set_error ([ce_string ("Invalid channel number")])
					Result := 0
					l_return := True
				elseif a_channel >= num_channels.item then
					sdl_set_error ([ce_string ("Invalid channel number")])
					Result := 0
					l_return := True
				end
				if not l_return then
					l_e := create {CE_POINTER_POINTER [PD_MIX_EFFECTINFO]}.make_shared ((mix_channel.item + a_channel).c_pointer + (create {PD_MIX_CHANNEL}.make).offset_of ("effects"))
				end
			end
			if not l_return then
				Result := (mix_remove_all_effects (a_channel, l_e))
			end
		end

	mix_unregister_all_effects (a_channel: INTEGER_32): INTEGER_32
		local
			l_retval: INTEGER_32
		do
			sdl_lock_audio
			l_retval := mix_unregister_all_effects_locked (a_channel)
			sdl_unlock_audio
			Result := l_retval
		end

feature {ANY} -- routine addresses

	mix_channels_address: CE_ROUTINE
		once ("PROCESS")
			create Result.make_agent (agent mix_channels)
		end

end
