class
	P_WAVESTREAM_STATIC

inherit
	CE_UTILITIES
	S_SDL_SDL_ERROR_STATIC
	S_STDLIB_STATIC
	S_STRING_STATIC
	S_SDL_SDL_AUDIO_STATIC
	S_SDL_SDL_RWOPS_STATIC

feature {ANY} -- public attributes

	music: CE_STATIC [CE_POINTER [PD_WAVSTREAM]]
		once ("PROCESS")
			Result.item := create {CE_CLASS_POINTER [PD_WAVSTREAM]}.make_shared (default_pointer)
		end

	mixer_0: CE_STATIC [SD_SDL_AUDIOSPEC]
		once ("PROCESS")
			Result.item := create {SD_SDL_AUDIOSPEC}.make
		end

	wavestream_volume: CE_STATIC [INTEGER_32]
		once ("PROCESS")
			Result.item := 128
		end

feature {ANY} -- public routines

	wavstream_init (a_mixerfmt: CE_POINTER [SD_SDL_AUDIOSPEC]): INTEGER_32
		do
			mixer_0.item := (a_mixerfmt.item).twin
			Result := 0
		end

	wavstream_set_volume (a_volume: INTEGER_32)
		do
			wavestream_volume.item := a_volume
		end

	wavstream_load_song_rw (a_rw: CE_POINTER [SD_SDL_RWOPS]; a_magic: CE_POINTER [INTEGER_8]; a_freerw: INTEGER_32): CE_POINTER [PD_WAVSTREAM]
		local
			l_wave: CE_POINTER [PD_WAVSTREAM]
			l_wavespec: CE_ARRAY [SD_SDL_AUDIOSPEC]
			l_return: BOOLEAN
		do
			create {CE_CLASS_ARRAY [SD_SDL_AUDIOSPEC]} l_wavespec.make (<<1>>)

			if not (mixer_0.item.format).to_boolean then
				sdl_set_error ([ce_string ("WAV music output not started")])
				if a_freerw.to_boolean then
					(a_rw.item.close).call ([a_rw])
				end
				Result := (create {CE_CLASS_POINTER [PD_WAVSTREAM]}.make_shared (default_pointer))
				l_return := True
			end
			if not l_return then
				l_wave := create {CE_CLASS_POINTER [PD_WAVSTREAM]}.make_cast ((malloc ((create {PD_WAVSTREAM}.make).structure_size.to_natural_32)))
				if (not l_wave.is_void) then
					memset (create {CE_VOID_POINTER}.make_cast (l_wave), 0, (create {PD_WAVSTREAM}.make).structure_size.to_natural_32).do_nothing
					l_wave.item.freerw := a_freerw.to_natural_32
					if (strcmp (a_magic, ce_string ("RIFF"))) = 0 then
						l_wave.item.rw := load_wavstream (a_rw, l_wavespec, create {CE_INTEGER_32_POINTER}.make_shared (l_wave.c_pointer + (create {PD_WAVSTREAM}.make).offset_of ("start")), create {CE_INTEGER_32_POINTER}.make_shared (l_wave.c_pointer + (create {PD_WAVSTREAM}.make).offset_of ("stop")))
					elseif (strcmp (a_magic, ce_string ("FORM"))) = 0 then
						l_wave.item.rw := load_aiffstream (a_rw, l_wavespec, create {CE_INTEGER_32_POINTER}.make_shared (l_wave.c_pointer + (create {PD_WAVSTREAM}.make).offset_of ("start")), create {CE_INTEGER_32_POINTER}.make_shared (l_wave.c_pointer + (create {PD_WAVSTREAM}.make).offset_of ("stop")))
					else
						sdl_set_error ([ce_string ("Unknown WAVE format")])
					end
					if (l_wave.item.rw).is_void then
						free (create {CE_VOID_POINTER}.make_cast (l_wave))
						if a_freerw.to_boolean then
							(a_rw.item.close).call ([a_rw])
						end
						Result := (create {CE_CLASS_POINTER [PD_WAVSTREAM]}.make_shared (default_pointer))
						l_return := True
					end
					if not l_return then
						sdl_build_audio_cvt (create {CE_CLASS_POINTER [SD_SDL_AUDIOCVT]}.make_shared (l_wave.c_pointer + (create {PD_WAVSTREAM}.make).offset_of ("cvt")), l_wavespec.item.format, l_wavespec.item.channels, l_wavespec.item.freq, mixer_0.item.format, mixer_0.item.channels, mixer_0.item.freq).do_nothing
					end
				else
					sdl_error ({SC_SDL_ERRORCODE}.sdl_enomem)
					if a_freerw.to_boolean then
						(a_rw.item.close).call ([a_rw])
					end
					Result := (create {CE_CLASS_POINTER [PD_WAVSTREAM]}.make_shared (default_pointer))
					l_return := True
				end
			end
			if not l_return then
				Result := l_wave
			end
		end

	wavstream_start (a_wave: CE_POINTER [PD_WAVSTREAM])
		do
			((a_wave.item.rw).item.seek).call ([a_wave.item.rw, (a_wave.item.start).to_integer_32, 0])
			music.item := a_wave
		end

	wavstream_play_some (a_stream: CE_POINTER [NATURAL_8]; a_len: INTEGER_32): INTEGER_32
		local
			l_len: INTEGER_32
			l_pos2: INTEGER_32
			l_left: INTEGER_32
			l_original_len: INTEGER_32
			l_worksize: INTEGER_32
			l_data: CE_POINTER [NATURAL_8]
			l_return: BOOLEAN
			l_aaf: CE_ALLOCA_FACTORY
		do
			l_len := a_len
			l_left := 0
			if (not music.item.is_void) then
				l_pos2 := (((music.item.item.rw).item.seek).integer_32_item ([music.item.item.rw, 0, 1])).to_integer_32
				if l_pos2 < music.item.item.stop then
					if (music.item.item.cvt.needed).to_boolean then
						l_original_len := (l_len.to_double / music.item.item.cvt.len_ratio).truncated_to_integer
						if music.item.item.cvt.len /= l_original_len then
							if not (music.item.item.cvt.buf).is_void then
								free (create {CE_VOID_POINTER}.make_cast (music.item.item.cvt.buf))
							end
							l_worksize := l_original_len * music.item.item.cvt.len_mult
							music.item.item.cvt.buf := create {CE_NATURAL_8_POINTER}.make_cast ((malloc (l_worksize.to_natural_32)))
							if (music.item.item.cvt.buf).is_void then
								Result := 0
								l_return := True
							end
							if not l_return then
								music.item.item.cvt.len := l_original_len
							end
						end
						if not l_return then
							if music.item.item.stop - l_pos2 < l_original_len.to_integer_32 then
								l_left := (l_original_len.to_integer_32 - (music.item.item.stop - l_pos2)).to_integer_32
								l_original_len := l_original_len - l_left
								l_left := (l_left.to_double * music.item.item.cvt.len_ratio).truncated_to_integer
							end
							l_original_len := ((music.item.item.rw).item.read).integer_32_item ([music.item.item.rw, create {CE_VOID_POINTER}.make_cast (music.item.item.cvt.buf), 1, l_original_len])
							if (((music.item.item.cvt.src_format).to_integer_32).bit_and (16)).to_boolean then
								if ((l_original_len).bit_and (1)).to_boolean then
									l_original_len := l_original_len - 1
								end
							end
							music.item.item.cvt.len := l_original_len
							sdl_convert_audio (create {CE_CLASS_POINTER [SD_SDL_AUDIOCVT]}.make_shared (music.item.c_pointer + (create {PD_WAVSTREAM}.make).offset_of ("cvt"))).do_nothing
							sdl_mix_audio (a_stream, music.item.item.cvt.buf, (music.item.item.cvt.len_cvt).to_natural_32, wavestream_volume.item)
						end
					else
						if music.item.item.stop - l_pos2 < l_len.to_integer_32 then
							l_left := (l_len.to_integer_32 - (music.item.item.stop - l_pos2)).to_integer_32
							l_len := l_len - l_left
						end
						l_data := create {CE_NATURAL_8_POINTER}.make_cast ((l_aaf.new ({PLATFORM}.Natural_8_bytes.to_natural_32 * l_len.to_natural_32)))
						if (not l_data.is_void) then
							((music.item.item.rw).item.read).call ([music.item.item.rw, create {CE_VOID_POINTER}.make_cast (l_data), l_len, 1])
							sdl_mix_audio (a_stream, l_data, l_len.to_natural_32, wavestream_volume.item)
						end
					end
				end
			end
			if not l_return then
				Result := l_left
			end
		end

	wavstream_stop
		do
			music.item := create {CE_CLASS_POINTER [PD_WAVSTREAM]}.make_shared (default_pointer)
		end

	wavstream_free_song (a_wave: CE_POINTER [PD_WAVSTREAM])
		do
			if (not a_wave.is_void) then
				if (not (a_wave.item.cvt.buf).is_void) then
					free (create {CE_VOID_POINTER}.make_cast (a_wave.item.cvt.buf))
				end
				if (a_wave.item.freerw).to_boolean then
					((a_wave.item.rw).item.close).call ([a_wave.item.rw])
				end
				free (create {CE_VOID_POINTER}.make_cast (a_wave))
			end
		end

	wavstream_active: INTEGER_32
		local
			l_active: INTEGER_32
		do
			l_active := 0
			if (not music.item.is_void) then
				if (((music.item.item.rw).item.seek).integer_32_item ([music.item.item.rw, 0, 1])).to_integer_32 < music.item.item.stop then
					l_active := 1
				end
			end
			Result := l_active
		end

	read_chunk (a_src: CE_POINTER [SD_SDL_RWOPS]; a_chunk: CE_POINTER [PD_CHUNK]; a_read_data: INTEGER_32): INTEGER_32
		local
			l_return: BOOLEAN
		do
			a_chunk.item.magic := sdl_read_le32 (a_src)
			a_chunk.item.length := sdl_read_le32 (a_src)
			if a_read_data.to_boolean then
				a_chunk.item.data := create {CE_NATURAL_8_POINTER}.make_cast ((malloc (a_chunk.item.length)))
				if (a_chunk.item.data).is_void then
					sdl_set_error ([ce_string ("Out of memory")])
					Result := (-1)
					l_return := True
				end
				if not l_return then
					if ((a_src.item.read).integer_32_item ([a_src, create {CE_VOID_POINTER}.make_cast (a_chunk.item.data), (a_chunk.item.length).to_integer_32, 1])) /= 1 then
						sdl_set_error ([ce_string ("Couldn%'t read chunk")])
						free (create {CE_VOID_POINTER}.make_cast (a_chunk.item.data))
						Result := (-1)
						l_return := True
					end
				end
			else
				(a_src.item.seek).call ([a_src, (a_chunk.item.length).to_integer_32, 1])
			end
			if not l_return then
				Result := (a_chunk.item.length).to_integer_32
			end
		end

	load_wavstream (a_src: CE_POINTER [SD_SDL_RWOPS]; a_spec: CE_POINTER [SD_SDL_AUDIOSPEC]; a_start, a_stop: CE_POINTER [INTEGER_32]): CE_POINTER [SD_SDL_RWOPS]
		local
			l_was_error: INTEGER_32
			l_chunk: CE_ARRAY [PD_CHUNK]
			l_lenread: INTEGER_32
			l_riffchunk: NATURAL_32
			l_wavelen: NATURAL_32
			l_wavemagic: NATURAL_32
			l_format: CE_POINTER [PD_WAVEFMT]
			l_return, l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			create {CE_CLASS_ARRAY [PD_CHUNK]} l_chunk.make (<<1>>)

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_format := create {CE_CLASS_POINTER [PD_WAVEFMT]}.make_shared (default_pointer)
					l_was_error := 0
					l_riffchunk := sdl_read_le32 (a_src)
					l_wavelen := sdl_read_le32 (a_src)
					l_wavemagic := sdl_read_le32 (a_src)
					if l_riffchunk /= {NATURAL_32} 1_179_011_410 then
						sdl_set_error ([ce_string ("Unrecognized file type (not WAVE)")])
						l_was_error := 1
						l_pos := -1 + 18 -- goto done
					elseif l_wavemagic /= {NATURAL_32} 1_163_280_727 then
						sdl_set_error ([ce_string ("Unrecognized file type (not WAVE)")])
						l_was_error := 1
						l_pos := -1 + 18 -- goto done
					end
				when 1 then
					l_chunk.item.data := create {CE_NATURAL_8_POINTER}.make_shared (default_pointer)
				when 2 then -- loop condition:
					if (l_break) then
						l_pos := -1 + 5 -- skip loop
					end
				when 3 then -- loop body:
						l_break := False
						if (not (l_chunk.item.data).is_void) then
							free (create {CE_VOID_POINTER}.make_cast (l_chunk.item.data))
						end
						l_lenread := read_chunk (a_src, l_chunk, 1)
						if l_lenread < 0 then
							l_was_error := 1
							l_pos := -1 + 18 -- goto done
						end
				when 4 then
						if l_chunk.item.magic /= {NATURAL_32} 1_952_670_054 then
							if l_chunk.item.magic /= {NATURAL_32} 1_414_744_396 then
								l_break := True
							end
						end
						l_pos := -1 + 2 -- back to loop condition
				when 5 then -- loop done
					l_break := False
					l_format := create {CE_CLASS_POINTER [PD_WAVEFMT]}.make_cast (l_chunk.item.data)
					if l_chunk.item.magic /= {NATURAL_32} 544_501_094 then
						free (create {CE_VOID_POINTER}.make_cast (l_chunk.item.data))
						sdl_set_error ([ce_string ("Complex WAVE files not supported")])
						l_was_error := 1
						l_pos := -1 + 18 -- goto done
					end
				when 6 then
					inspect (l_format.item.encoding).to_integer_32 -- switch 1 head
					when 1 then l_pos := -1 + 7
					else l_pos := -1 + 8
					end
				when 7 then -- switch 1, case 1
					l_pos := -1 + 9 -- switch 1; break
				when 8 then -- switch 1, default:
					sdl_set_error ([ce_string ("Unknown WAVE data format")])
					l_was_error := 1
					l_pos := -1 + 18 -- goto done
				when 9 then -- switch 1, done:
					memset (create {CE_VOID_POINTER}.make_cast (a_spec), 0, (create {SD_SDL_AUDIOSPEC}.make).structure_size.to_natural_32).do_nothing
					a_spec.item.freq := (l_format.item.frequency).to_integer_32
					inspect (l_format.item.bitspersample).to_integer_32 -- switch 2 head
					when 8 then l_pos := -1 + 10
					when 16 then l_pos := -1 + 11
					else l_pos := -1 + 12
					end
				when 10 then -- switch 2, case 8
					a_spec.item.format := (8).to_natural_16
					l_pos := -1 + 13 -- switch 2; break
				when 11 then -- switch 2, case 16
					a_spec.item.format := (32_784).to_natural_16
					l_pos := -1 + 13 -- switch 2; break
				when 12 then -- switch 2, default:
					sdl_set_error ([ce_string ("Unknown PCM data format")])
					l_was_error := 1
					l_pos := -1 + 18 -- goto done
				when 13 then -- switch 2, done:
					a_spec.item.channels := (l_format.item.channels).to_natural_8
					a_spec.item.samples := (4_096).to_natural_16
					l_chunk.item.data := create {CE_NATURAL_8_POINTER}.make_shared (default_pointer)
				when 14 then -- loop condition:
					if (l_break) then
						l_pos := -1 + 17 -- skip loop
					end
				when 15 then -- loop body:
						l_break := False
						a_start.item := (((a_src.item.seek).integer_32_item ([a_src, 0, 1])).to_natural_32 + {NATURAL_32} 2 * {PLATFORM}.Natural_32_bytes.to_natural_32).to_integer_32
						l_lenread := read_chunk (a_src, l_chunk, 0)
						if l_lenread < 0 then
							l_was_error := 1
							l_pos := -1 + 18 -- goto done
						end
				when 16 then
						if l_chunk.item.magic = {NATURAL_32} 1_635_017_060 then
							l_break := True
						end
						l_pos := -1 + 14 -- back to loop condition
				when 17 then -- loop done
					l_break := False
					a_stop.item := ((a_src.item.seek).integer_32_item ([a_src, 0, 1])).to_integer_32
				when 18 then -- done:
					if not l_format.is_void then
						free (create {CE_VOID_POINTER}.make_cast (l_format))
					end
					if l_was_error.to_boolean then
						Result := (create {CE_CLASS_POINTER [SD_SDL_RWOPS]}.make_shared (default_pointer))
						l_return := True
					end
					if not l_return then
						Result := a_src
					end
				when 19 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	sane_to_uint32_0 (a_sanebuf: CE_POINTER [NATURAL_8]): NATURAL_32
		local
			l_return: BOOLEAN
		do
			if (((a_sanebuf[0]).to_integer_32).bit_and (128)).to_boolean then
				Result := (0).to_natural_32
				l_return := True
			end
			if not l_return and then ((a_sanebuf[0]).to_integer_32 <= 63) then
				Result := (1).to_natural_32
				l_return := True
			end
			if not l_return and then ((a_sanebuf[0]).to_integer_32 > 64) then
				Result := (67_108_864).to_natural_32
				l_return := True
			end
			if not l_return and then ((a_sanebuf[0]).to_integer_32 = 64) then
				if (a_sanebuf[1]).to_integer_32 > 28 then
					Result := (800_000_000).to_natural_32
					l_return := True
				end
			end
			if not l_return then
				Result := (((((((((((a_sanebuf[2]).to_integer_32).bit_shift_left (23))).bit_or ((((a_sanebuf[3]).to_integer_32).bit_shift_left (15))))).bit_or ((((a_sanebuf[4]).to_integer_32).bit_shift_left (7))))).bit_or ((((a_sanebuf[5]).to_integer_32).bit_shift_right (1))))).bit_shift_right ((29 - (a_sanebuf[1]).to_integer_32))).to_natural_32
			end
		end

	load_aiffstream (a_src: CE_POINTER [SD_SDL_RWOPS]; a_spec: CE_POINTER [SD_SDL_AUDIOSPEC]; a_start, a_stop: CE_POINTER [INTEGER_32]): CE_POINTER [SD_SDL_RWOPS]
		local
			l_was_error: INTEGER_32
			l_found_ssnd: INTEGER_32
			l_found_comm: INTEGER_32
			l_chunk_type: NATURAL_32
			l_chunk_length: NATURAL_32
			l_next_chunk: INTEGER_32
			l_formchunk: NATURAL_32
			l_aiffmagic: NATURAL_32
			l_offset: NATURAL_32
			l_blocksize: NATURAL_32
			l_channels: NATURAL_16
			l_numsamples: NATURAL_32
			l_samplesize: NATURAL_16
			l_sane_freq: CE_ARRAY [NATURAL_8]
			l_frequency: NATURAL_32
			l_return, l_break: BOOLEAN
			l_pos: INTEGER_32
		do
			create {CE_NATURAL_8_ARRAY} l_sane_freq.make (<<10>>)

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_channels := (0).to_natural_16
					l_numsamples := (0).to_natural_32
					l_samplesize := (0).to_natural_16
					l_frequency := (0).to_natural_32
					l_was_error := 0
					l_formchunk := sdl_read_le32 (a_src)
					l_chunk_length := sdl_read_be32 (a_src)
					l_aiffmagic := sdl_read_le32 (a_src)
					if l_formchunk /= {NATURAL_32} 1_297_239_878 then
						sdl_set_error ([ce_string ("Unrecognized file type (not AIFF)")])
						l_was_error := 1
						l_pos := -1 + 23 -- goto done
					elseif l_aiffmagic /= {NATURAL_32} 1_179_011_393 then
						sdl_set_error ([ce_string ("Unrecognized file type (not AIFF)")])
						l_was_error := 1
						l_pos := -1 + 23 -- goto done
					end
				when 1 then
					l_found_ssnd := 0
					l_found_comm := 0
				when 2 then -- loop condition:
					if (l_break) then
						l_pos := -1 + 16 -- skip loop
					end
				when 3 then -- loop body:
						l_break := False
						l_chunk_type := sdl_read_le32 (a_src)
						l_chunk_length := sdl_read_be32 (a_src)
						l_next_chunk := (((a_src.item.seek).integer_32_item ([a_src, 0, 1])).to_natural_32 + l_chunk_length).to_integer_32
						if l_chunk_length = {NATURAL_32} 0 then
							l_break := True
						end
						if (l_break) then
							l_pos := -1 + 15 -- skip if block
						end
				when 4 then -- if:
						inspect l_chunk_type -- switch 1 head
						when 1145983827 then l_pos := -1 + 5
						when 1296912195 then l_pos := -1 + 6
						else l_pos := -1 + 7
						end
				when 5 then -- switch 1, case 1145983827u
						l_found_ssnd := 1
						l_offset := sdl_read_be32 (a_src)
						l_blocksize := sdl_read_be32 (a_src)
						a_start.item := (((a_src.item.seek).integer_32_item ([a_src, 0, 1])).to_natural_32 + l_offset).to_integer_32
						l_pos := -1 + 8 -- switch 1; break
				when 6 then -- switch 1, case 1296912195u
						l_found_comm := 1
						l_channels := sdl_read_be16 (a_src)
						l_numsamples := sdl_read_be32 (a_src)
						l_samplesize := sdl_read_be16 (a_src);
						(a_src.item.read).call ([a_src, create {CE_VOID_POINTER}.make_cast (l_sane_freq), ({PLATFORM}.Natural_8_bytes.to_natural_32 * (10).to_natural_32).to_integer_32, 1])
						l_frequency := sane_to_uint32_0 (l_sane_freq)
						l_pos := -1 + 8 -- switch 1; break
				when 7 then -- switch 1, default:
						l_pos := -1 + 8 -- switch 1; break
				when 8 then -- switch 1, done:
						if l_found_ssnd.to_boolean then
							l_pos := -1 + 11 -- skip if block
						end
				when 9 then -- if:
						l_pos := -1 + 12 -- goto _L
				when 10 then -- if block done, skip else block:
						l_pos := -1 + 15
				when 11 then -- else:
						if l_found_comm.to_boolean then
							l_pos := -1 + 14 -- skip if block
						end
				when 12 then -- if: | _L:
						if ((a_src.item.seek).integer_32_item ([a_src, l_next_chunk.to_integer_32, 0])) = -1 then
							l_break := True
						end
				when 13 then -- if block done, skip else block:
						l_pos := -1 + 15
				when 14 then -- else:
						l_break := True
				when 15 then -- if done | if done
						l_pos := -1 + 2 -- back to loop condition
				when 16 then -- loop done
					l_break := False
					if not l_found_ssnd.to_boolean then
						sdl_set_error ([ce_string ("Bad AIFF file (no SSND chunk)")])
						l_was_error := 1
						l_pos := -1 + 23 -- goto done
					end
				when 17 then
					if not l_found_comm.to_boolean then
						sdl_set_error ([ce_string ("Bad AIFF file (no COMM chunk)")])
						l_was_error := 1
						l_pos := -1 + 23 -- goto done
					end
				when 18 then
					a_stop.item := ((a_start.item).to_natural_32 + ((l_channels.to_natural_32 * l_numsamples) * (l_samplesize.to_integer_32 // 8).to_natural_32).to_natural_32).to_integer_32
					memset (create {CE_VOID_POINTER}.make_cast (a_spec), 0, (create {SD_SDL_AUDIOSPEC}.make).structure_size.to_natural_32).do_nothing
					a_spec.item.freq := l_frequency.to_integer_32
					inspect l_samplesize.to_integer_32 -- switch 2 head
					when 8 then l_pos := -1 + 19
					when 16 then l_pos := -1 + 20
					else l_pos := -1 + 21
					end
				when 19 then -- switch 2, case 8
					a_spec.item.format := (32_776).to_natural_16
					l_pos := -1 + 22 -- switch 2; break
				when 20 then -- switch 2, case 16
					a_spec.item.format := (36_880).to_natural_16
					l_pos := -1 + 22 -- switch 2; break
				when 21 then -- switch 2, default:
					sdl_set_error ([ce_string ("Unknown samplesize in data format")])
					l_was_error := 1
					l_pos := -1 + 23 -- goto done
				when 22 then -- switch 2, done:
					a_spec.item.channels := l_channels.to_natural_8
					a_spec.item.samples := (4_096).to_natural_16
				when 23 then -- done:
					if l_was_error.to_boolean then
						Result := (create {CE_CLASS_POINTER [SD_SDL_RWOPS]}.make_shared (default_pointer))
						l_return := True
					end
					if not l_return then
						Result := a_src
					end
				when 24 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

end
