class
	S_STDLIB_STATIC

inherit
	CE_UTILITIES

feature {ANY} -- public routines

	exit (a_code: INTEGER_32)
		external
			"C inline use <sdl_mixer.h>"
		alias
			"_exit($a_code);"
		end

	exit2 (a_status: INTEGER_32)
		do
			exit (a_status)
		end

	abs (a_x: INTEGER_32): INTEGER_32
		external
			"C inline use <sdl_mixer.h>"
		alias
			"return abs($a_x);"
		end

	i64toa (a_val: INTEGER_64; a_dst_buf: CE_POINTER [INTEGER_8]; a_radix: INTEGER_32): CE_POINTER [INTEGER_8]
		do
			create {CE_INTEGER_8_POINTER} Result.make_shared (c_i64toa (a_val, a_dst_buf.c_pointer, a_radix))
		end

	c_i64toa (a_val: INTEGER_64; a_dst_buf: POINTER; a_radix: INTEGER_32): POINTER
		external
			"C inline use <sdl_mixer.h>"
		alias
			"return (EIF_POINTER)_i64toa($a_val, $a_dst_buf, $a_radix);"
		end

	ui64toa (a_val: NATURAL_64; a_dst_buf: CE_POINTER [INTEGER_8]; a_radix: INTEGER_32): CE_POINTER [INTEGER_8]
		do
			create {CE_INTEGER_8_POINTER} Result.make_shared (c_ui64toa (a_val, a_dst_buf.c_pointer, a_radix))
		end

	c_ui64toa (a_val: NATURAL_64; a_dst_buf: POINTER; a_radix: INTEGER_32): POINTER
		external
			"C inline use <sdl_mixer.h>"
		alias
			"return (EIF_POINTER)_ui64toa($a_val, $a_dst_buf, $a_radix);"
		end

	atoi64 (a_string: CE_POINTER [INTEGER_8]): INTEGER_64
		do
			Result := c_atoi64 (a_string.c_pointer)
		end

	c_atoi64 (a_string: POINTER): INTEGER_64
		external
			"C inline use <sdl_mixer.h>"
		alias
			"return _atoi64($a_string);"
		end

	free (a_memory: CE_POINTER [ANY])
		do
			c_free (a_memory.c_pointer)
		end

	c_free (a_memory: POINTER)
		external
			"C inline use <sdl_mixer.h>"
		alias
			"free($a_memory);"
		end

	malloc (a_size: NATURAL_32): CE_POINTER [ANY]
		do
			create {CE_VOID_POINTER} Result.make_shared_with_count (
					default_pointer.memory_alloc (a_size.to_integer_32),
					a_size.to_integer_32
				)
		end

	realloc (a_ptr: CE_POINTER [ANY]; a_size: NATURAL_32): CE_POINTER [ANY]
		do
			create {CE_VOID_POINTER} Result.make_shared_with_count (
					a_ptr.c_pointer.memory_realloc (a_size.to_integer_32),
					a_size.to_integer_32
				)
		end

	i64tow (a_val: INTEGER_64; a_dst_buf: CE_POINTER [NATURAL_16]; a_radix: INTEGER_32): CE_POINTER [NATURAL_16]
		do
			create {CE_NATURAL_16_POINTER} Result.make_shared (c_i64tow (a_val, a_dst_buf.c_pointer, a_radix))
		end

	c_i64tow (a_val: INTEGER_64; a_dst_buf: POINTER; a_radix: INTEGER_32): POINTER
		external
			"C inline use <sdl_mixer.h>"
		alias
			"return (EIF_POINTER)_i64tow($a_val, $a_dst_buf, $a_radix);"
		end

	ui64tow (a_val: NATURAL_64; a_dst_buf: CE_POINTER [NATURAL_16]; a_radix: INTEGER_32): CE_POINTER [NATURAL_16]
		do
			create {CE_NATURAL_16_POINTER} Result.make_shared (c_ui64tow (a_val, a_dst_buf.c_pointer, a_radix))
		end

	c_ui64tow (a_val: NATURAL_64; a_dst_buf: POINTER; a_radix: INTEGER_32): POINTER
		external
			"C inline use <sdl_mixer.h>"
		alias
			"return (EIF_POINTER)_ui64tow($a_val, $a_dst_buf, $a_radix);"
		end

	wtoi64 (a_str: CE_POINTER [NATURAL_16]): INTEGER_64
		do
			Result := c_wtoi64 (a_str.c_pointer)
		end

	c_wtoi64 (a_str: POINTER): INTEGER_64
		external
			"C inline use <sdl_mixer.h>"
		alias
			"return _wtoi64($a_str);"
		end

	llabs (a_j: INTEGER_64): INTEGER_64
		local
			l_tmp: INTEGER_64
		do
			if a_j >= {INTEGER_64} 0 then
				l_tmp := a_j
			else
				l_tmp := -a_j
			end
			Result := l_tmp
		end

	atoll (a_c: CE_POINTER [INTEGER_8]): INTEGER_64
		do
			Result := (atoi64 (a_c))
		end

	lltoa (a_n: INTEGER_64; a_c: CE_POINTER [INTEGER_8]; a_i: INTEGER_32): CE_POINTER [INTEGER_8]
		do
			Result := (i64toa (a_n, a_c, a_i))
		end

	ulltoa (a_n: NATURAL_64; a_c: CE_POINTER [INTEGER_8]; a_i: INTEGER_32): CE_POINTER [INTEGER_8]
		do
			Result := (ui64toa (a_n, a_c, a_i))
		end

	wtoll (a_w: CE_POINTER [NATURAL_16]): INTEGER_64
		do
			Result := (wtoi64 (a_w))
		end

	lltow (a_n: INTEGER_64; a_w: CE_POINTER [NATURAL_16]; a_i: INTEGER_32): CE_POINTER [NATURAL_16]
		do
			Result := (i64tow (a_n, a_w, a_i))
		end

	ulltow (a_n: NATURAL_64; a_w: CE_POINTER [NATURAL_16]; a_i: INTEGER_32): CE_POINTER [NATURAL_16]
		do
			Result := (ui64tow (a_n, a_w, a_i))
		end

	calloc (a_nmemb: NATURAL_32; a_size: NATURAL_32): CE_POINTER [ANY]
		do
			create {CE_VOID_POINTER} Result.make_shared_with_count (
					default_pointer.memory_calloc (a_nmemb.to_integer_32, a_size.to_integer_32),
					(a_nmemb * a_size).to_integer_32
				)
		end

end
