class
	SD_HIDDEN

inherit
	CE_UTILITIES
		redefine
			copy,
			is_equal
		end
	CE_CLASS
		redefine
			make,
			make_shared,
			make_initialized,
			structure_size,
			offset_of,
			copy,
			is_equal
		end

create
	make,
	make_shared,
	make_initialized

feature {NONE}

	make
		do
			internal_c_pointer := internal_c_pointer.memory_alloc (structure_size)
			is_allocated_memory := True
		end

	make_shared (a_c_pointer: POINTER)
		do
			internal_c_pointer := a_c_pointer
		end

	make_initialized (a_data: TUPLE [
				a_win32io: SD_WIN32IO
				a_stdio: SD_STDIO
				a_mem: SD_MEM
				a_unknown: SD_UNKNOWN
			])
		require else
			a_data /= Void
		do
			make
			win32io.c_pointer.memory_copy (a_data.a_win32io.c_pointer, a_data.a_win32io.structure_size)
			stdio.c_pointer.memory_copy (a_data.a_stdio.c_pointer, a_data.a_stdio.structure_size)
			mem.c_pointer.memory_copy (a_data.a_mem.c_pointer, a_data.a_mem.structure_size)
			unknown.c_pointer.memory_copy (a_data.a_unknown.c_pointer, a_data.a_unknown.structure_size)
		end

feature {ANY} -- fields

	win32io: SD_WIN32IO
		once ("OBJECT")
			create Result.make_shared (c__win32io (internal_c_pointer))
		end

	stdio: SD_STDIO
		once ("OBJECT")
			create Result.make_shared (c__stdio (internal_c_pointer))
		end

	mem: SD_MEM
		once ("OBJECT")
			create Result.make_shared (c__mem (internal_c_pointer))
		end

	unknown: SD_UNKNOWN
		once ("OBJECT")
			create Result.make_shared (c__unknown (internal_c_pointer))
		end

feature {ANY}

	structure_size: INTEGER_32
		external
			"C inline use %"sdl_mixer.h%""
		alias
			"return sizeof(union __anonunion_hidden_13);"
		end

	structure_alignment: INTEGER_32
		external
			"C inline use %"sdl_mixer.h%""
		alias
			"return __alignof__(union __anonunion_hidden_13);"
		end

	offset_of (a_field_name: STRING_8): INTEGER_32
		local
			l_c_str: C_STRING
		do
			create l_c_str.make (a_field_name)
			Result := c__offset_of (l_c_str.item)
		end

feature {ANY}

	copy (a_other: like Current)
		do
			make
			internal_c_pointer.memory_copy (a_other.c_pointer, structure_size)
		end

	is_equal (a_other: like Current): BOOLEAN
		do
			Result := internal_c_pointer.memory_compare (a_other.c_pointer, structure_size)
		end

feature {NONE}

	structure_info: CE_STRUCTURE
		do
			-- dummy for CE_CLASS invariant
			create Result.make_union (<<>>)
			Result.alignment := structure_alignment
		end

	c__win32io (a_c_pointer: POINTER): POINTER
		external
			"C inline use %"sdl_mixer.h%""
		alias
			"return (EIF_POINTER)&((union __anonunion_hidden_13 *)$a_c_pointer)->win32io;"
		end

	c__stdio (a_c_pointer: POINTER): POINTER
		external
			"C inline use %"sdl_mixer.h%""
		alias
			"return (EIF_POINTER)&((union __anonunion_hidden_13 *)$a_c_pointer)->stdio;"
		end

	c__mem (a_c_pointer: POINTER): POINTER
		external
			"C inline use %"sdl_mixer.h%""
		alias
			"return (EIF_POINTER)&((union __anonunion_hidden_13 *)$a_c_pointer)->mem;"
		end

	c__unknown (a_c_pointer: POINTER): POINTER
		external
			"C inline use %"sdl_mixer.h%""
		alias
			"return (EIF_POINTER)&((union __anonunion_hidden_13 *)$a_c_pointer)->unknown;"
		end

	c__offset_of (a_field_name: POINTER): INTEGER_32
		external
			"C inline use %"sdl_mixer.h%", <string.h>, <stddef.h>, <stdio.h>"
		alias
			"[
				if(strcmp($a_field_name, "win32io") == 0) return offsetof(union __anonunion_hidden_13, win32io);
				if(strcmp($a_field_name, "stdio") == 0) return offsetof(union __anonunion_hidden_13, stdio);
				if(strcmp($a_field_name, "mem") == 0) return offsetof(union __anonunion_hidden_13, mem);
				if(strcmp($a_field_name, "unknown") == 0) return offsetof(union __anonunion_hidden_13, unknown);
				fprintf(stderr, "union __anonunion_hidden_13.%s not found\n", $a_field_name);
				return -1;
			]"
		end

end
