class
	DEMO_WINDOW

inherit
	EV_TITLED_WINDOW
	P_MIXER_STATIC
		undefine
			default_create,
			copy
		end
	P_MUSIC_STATIC
		undefine
			default_create,
			copy
		end

create
	make

feature {NONE}

	make
		local
			l_volume_control: EV_HORIZONTAL_RANGE
			l_file_box, l_controls_box, l_volume_box: EV_HORIZONTAL_BOX
			l_main_box: EV_VERTICAL_BOX
		do
			make_with_title ("Awesome Music Player")

			create file_info
			file_info.disable_edit
			file_info.set_minimum_width (250)

			create play_pause_button.make_with_text_and_action ("Play", agent play_pause_clicked)
			play_pause_button.disable_sensitive

			create stop_button.make_with_text_and_action ("Stop", agent stop_clicked)
			stop_button.disable_sensitive

			create l_volume_control.make_with_value_range (0 |..| 128)
			l_volume_control.set_value (mix_volume_music (-1))
			l_volume_control.change_actions.extend (agent volume_changed)
			l_volume_control.set_minimum_width (100)

			create l_file_box
			add_widget_to_box (l_file_box, create {EV_LABEL}.make_with_text ("File:"), False)
			add_widget_to_box (l_file_box, file_info, True)
			add_widget_to_box (l_file_box, create {EV_BUTTON}.make_with_text_and_action ("Open", agent open_clicked), False)

			create l_controls_box
			add_widget_to_box (l_controls_box, create {EV_HORIZONTAL_BOX}, True)
			add_widget_to_box (l_controls_box, play_pause_button, False)
			add_widget_to_box (l_controls_box, stop_button, False)
			add_widget_to_box (l_controls_box, create {EV_HORIZONTAL_BOX}, True)

			create l_volume_box
			add_widget_to_box (l_volume_box, create {EV_HORIZONTAL_BOX}, True)
			add_widget_to_box (l_volume_box, create {EV_LABEL}.make_with_text ("Volume:"), False)
			add_widget_to_box (l_volume_box, l_volume_control, False)
			add_widget_to_box (l_volume_box, create {EV_HORIZONTAL_BOX}, True)

			create l_main_box
			add_widget_to_box (l_main_box, l_file_box, False)
			add_widget_to_box (l_main_box, l_controls_box, False)
			add_widget_to_box (l_main_box, create {EV_HORIZONTAL_SEPARATOR}, False)
			add_widget_to_box (l_main_box, l_volume_box, False)
			l_main_box.set_padding (3)

			extend (l_main_box)
			close_request_actions.extend (agent close_clicked)
		end

	add_widget_to_box (a_box: EV_BOX; a_widget: EV_WIDGET; a_expand: BOOLEAN)
		require
			a_box /= Void
			a_widget /= Void
		do
			a_box.extend (a_widget)
			if not a_expand then
				a_box.disable_item_expand (a_widget)
			end
		end

	show_error_dialog (a_title, a_message: STRING_8)
		require
			a_title /= Void and then not a_title.is_empty
			a_message /= Void and then not a_message.is_empty
		local
			l_dialog: EV_INFORMATION_DIALOG
		do
			create l_dialog.make_with_text (a_message)
			l_dialog.set_title (a_title)
			l_dialog.show_modal_to_window (Current)
		end

feature {NONE}

	file_info: EV_TEXT_FIELD

	play_pause_button, stop_button: EV_BUTTON

	audio_open: BOOLEAN

	music_file: MUSIC_FILE

feature {NONE}

	open_clicked
		local
			l_file_dialog: EV_FILE_OPEN_DIALOG
		do
			if not audio_open then
				if mix_open_audio (44100, 32784, 2, 1024) /= -1 then
					audio_open := True
					mix_hook_music_finished (create {CE_ROUTINE}.make_agent (agent music_finished))
				else
					show_error_dialog ("Opening Failed", "Opening the audio system failed:%N" + eif_string (sdl_get_error))
				end
			end

			if audio_open then
				create l_file_dialog.make_with_title ("Open Music File")
				l_file_dialog.filters.extend (["*.mp3;*.wav", "Music Files (*.mp3, *.wav)"])
				l_file_dialog.filters.extend (["*.*", "All Files"])
				l_file_dialog.show_modal_to_window (Current)

				if not l_file_dialog.file_name.is_empty then
					mix_halt_music.do_nothing
					create music_file.make (l_file_dialog.file_name)
					if
						music_file.music /= Void and then
						mix_play_music (music_file.music, 1) /= -1
					then
						file_info.set_text (music_file.file_name)
						play_pause_button.set_text ("Pause")
						play_pause_button.enable_sensitive
						stop_button.enable_sensitive
					else
						file_info.set_text ("")
						play_pause_button.set_text ("Play")
						play_pause_button.disable_sensitive
						stop_button.disable_sensitive
						music_file := Void

						show_error_dialog ("Opening Failed", "Opening the file failed:%N" + eif_string (sdl_get_error))
					end
				end
			end
		end

	play_pause_clicked
		require
			audio_open
			music_file /= Void
		do
			if not mix_playing_music.to_boolean then
				if mix_play_music (music_file.music, 1) /= -1 then
					play_pause_button.set_text ("Pause")
					stop_button.enable_sensitive
				else
					show_error_dialog ("Opening Failed", "Opening the file failed:%N" + eif_string (sdl_get_error))
				end
			else
				if mix_paused_music.to_boolean then
					mix_resume_music
					play_pause_button.set_text ("Pause")
				else
					mix_pause_music
					play_pause_button.set_text ("Play")
				end
			end
		end

	stop_clicked
		require
			audio_open
			music_file /= Void
		do
			mix_halt_music.do_nothing
			play_pause_button.set_text ("Play")
			stop_button.disable_sensitive
		end

	volume_changed (a_new_volume: INTEGER_32)
		do
			mix_volume_music (a_new_volume).do_nothing
		end

	close_clicked
		do
			if audio_open then
				mix_close_audio
			end

			destroy;
			environment_i.application.destroy
		end

	music_finished
		do
			environment_i.application.do_once_on_idle (agent
				do
					play_pause_button.set_text ("Play")
					stop_button.disable_sensitive
				end)
		end

invariant
	file_info /= Void
	play_pause_button /= Void
	stop_button /= Void
	music_file /= Void implies music_file.music /= Void

end
